import db from "../../models";
import { CronJob } from 'cron';
import { Op } from 'sequelize';
import path from 'path';
import request from 'request';
import fs from 'fs';
import { Utils } from "../../utills/helperMethods";
import moment from 'moment';
import Async from "async";
import { FolderNames, IdenfyStatus, ImageExentions, MediaSources, MediaTypes } from '../../utills/enums';
import { handleFileDelete } from './fsFileUpload';
import { Logging } from './logging.service';
import { sendMail } from "../../utills/mailer";
import { Constants } from "../../utills/constants";
import { checkVerificationDVS, sexOffenderScreening } from "./dvsService";
import { getIdenfyData, getIdenfyFiles, getIdenfyStatus } from "./idenfyService";
import { v4 } from "uuid";
import datingPageMediaService from "../routes/datingPageMedia/datingPageMedia.service";

const Story = db.Story;
const StoryViews = db.StoryViews;
const Comment = db.Comment;
const Reaction = db.Reaction;
const User = db.User;
const EmergencyContact = db.EmergencyContact;
const UserTimeLogs = db.UserTimeLogs;
const DocumentVerificationLogs = db.DocumentVerificationLogs;
const IdenfyLogs = db.IdenfyLogs;
const SignUpCheckList = db.SignUpCheckList;


// Cron at every midnight...
const storyCron = new CronJob('0 0 * * *', async () => {
    try {
        const expiredStories = await Story.findAll({
            where: {
                expire: {
                    [Op.lte]: moment().unix()
                }
            },
            raw: true
        });
        if (expiredStories.length > 0) {
            expiredStories.map(async (story) => {
                await StoryViews.destroy({
                    where: {
                        StoryId: story.id
                    }
                });
                await Comment.destroy({
                    where: {
                        StoryId: story.id
                    }
                });
                await Reaction.destroy({
                    where: {
                        StoryId: story.id
                    }
                });
                const storyDeleteLoad = {
                    type: MediaSources.STORY,
                    userId: story.UserId,
                    itemId: null,
                    filePath: story.mediaPath.split('/').pop()
                };

                const thumbnailDeleteLoad = {
                    type: MediaSources.STORY,
                    userId: story.UserId,
                    itemId: null,
                    filePath: story.thumbnail.split('/').pop()
                };

                await handleFileDelete(storyDeleteLoad);
                await handleFileDelete(thumbnailDeleteLoad);

                await Story.destroy({
                    where: {
                        id: story.id
                    }
                });
            })
        } else {
            Logging.logMessage('No Stories need to delete.')
        }
    } catch (e) {
        Logging.logGeneralError(e);
    }
});

// Cron at every midnight...
const datePageViewCountCron = new CronJob('0 0 * * *', async () => {
    try {
        const users = await User.findAll({
            attributes: [
                "id"
            ],
            raw: true
        });
        if (users.length > 0) {
            users.map(async (user) => {
                await User.update({ datePageViewCount: 0 }, {
                    where: { id: user.id }
                });
            });
        } else {
            Logging.logMessage('No Stories need to delete.')
        }
    } catch (e) {
        Logging.logGeneralError(e);
    }
});

// Cron at every 10 mins...
const alertSystemCron = new CronJob('*/10 * * * *', async () => {
    try {
        const usersOnDate = await User.findAll({
            where: {
                isOnDate: 1
            },
            attributes: [
                "id",
                "userName",
                "firstName",
                "lastName",
                "onDateLat",
                "onDateLong",
            ],
            raw: true
        });

        if (usersOnDate.length > 0) {
            usersOnDate.map(async (user) => {
                let lastTimeLog = await UserTimeLogs.findOne({
                    where: {
                        userId: user.id
                    },
                    raw: true
                });

                if (lastTimeLog && moment().diff(moment.unix(lastTimeLog.lastTimeStamp), "hours") >= 1) {
                    let alertContacts = await EmergencyContact.findAll({
                        where: {
                            UserId: user.id
                        },
                        raw: true
                    });
                    alertContacts.map(async (contact) => {
                        const sendTo = contact.email;
                        const fromName = user.firstName || (user.fistName && user.lastName) ? `${user.firstName} ${user.lastName || ""}` : user.userName;
                        const toName = contact.name;
                        const dateLocation = {
                            latitude: user.onDateLat,
                            longitude: user.onDateLong,
                        };
                        const currenctLocation = {
                            latitude: lastTimeLog.latitude,
                            longitude: lastTimeLog.longitude,
                        };
                        const sendMsg = Utils.getDateEmegencyAlertHtml(toName, fromName, dateLocation, currenctLocation);

                        await sendMail(Constants.FROM_EMAIL, sendTo, `Emergency Alert from ${fromName}`, sendMsg);
                    })

                    await UserTimeLogs.destroy({
                        where: {
                            userId: user.id
                        }
                    });
                }
            });
        }
    } catch (e) {
        Logging.logGeneralError(e);
    }
});

// Cron at every 5 mins...
const dvsVerificationCheckCron = new CronJob('*/5 * * * *', async () => {
    try {
        const unVerifiedDocs = await DocumentVerificationLogs.findAll({
            where: {
                succeeded: 0,
                verificationFailed: 0
            },
            raw: true
        });

        Async.each(unVerifiedDocs, async (documents) => {
            const checkedData = await checkVerificationDVS(documents["verificationId"]);
            if (checkedData && checkedData["request"]["status"] == 1) {
                if (checkedData['verificationResult']['faceVerificationResult']["confidence"] > Constants.DEFAULT_FACE_CONFIDANCE) {
                    await User.update({ isDocumentVerified: 1 }, {
                        where: { id: documents["userId"] }
                    });
                    await DocumentVerificationLogs.update({ succeeded: 1 }, {
                        where: {
                            verificationId: documents["verificationId"]
                        }
                    })
                } else {
                    await DocumentVerificationLogs.update({ succeeded: 1, verificationFailed: 1 }, {
                        where: {
                            verificationId: documents["verificationId"]
                        }
                    })
                }
            }
        });

    } catch (e) {
        Logging.logGeneralError(e);
    }
});

// Cron at every 5 mins...
const idenfyVerificationCheckCron = new CronJob('*/5 * * * *', async () => {
    try {
        const unVerifiedLogs = await IdenfyLogs.findAll({
            where: {
                succeed: 0
            },
            raw: true
        });

        Async.each(unVerifiedLogs, async (log: any) => {
            try {
                const documentStatus: any = await getIdenfyStatus(log.scanRef);
                if (documentStatus && documentStatus.status) {
                    await IdenfyLogs.update({ verificationStatus: documentStatus.status }, {
                        where: {
                            authToken: log.authToken
                        }
                    });

                    if (documentStatus.status == IdenfyStatus.APPROVED) {
                        const idenfyData: any = await getIdenfyData(log.scanRef);
                        const idenfyFiles: any = await getIdenfyFiles(log.scanRef);
                        const update: any = {};

                        if (idenfyData.docFirstName) update.firstName = idenfyData.docFirstName;
                        if (idenfyData.docLastName) update.lastName = idenfyData.docLastName;
                        if (idenfyData.docDob) update.DOB = idenfyData.docDob;
                        if (idenfyData.docSex) update.gender = idenfyData.docSex;

                        if (Object.keys(update).length > 0) {
                            await User.update(update, {
                                where: {
                                    id: log.userId
                                }
                            });
                        }

                        if (idenfyFiles && idenfyFiles.FACE && idenfyFiles.FACE != "") {
                            const destination = path.join(__dirname, '../../public/users/' + log.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/');
                            const profilePicDestination = path.join(__dirname, '../../public/users/' + log.userId + '/' + FolderNames.PROFILE_PICTURE + '/');
                            if (!fs.existsSync(destination)) {
                                fs.mkdirSync(destination, { recursive: true })
                            }

                            if (!fs.existsSync(profilePicDestination)) {
                                fs.mkdirSync(profilePicDestination, { recursive: true })
                            }

                            const fileName = `${v4()}${ImageExentions.PNG}`;
                            const profilePicName = `${v4()}${ImageExentions.PNG}`;

                            await request(idenfyFiles.FACE).pipe(fs.createWriteStream(`${destination}${fileName}`));
                            await request(idenfyFiles.FACE).pipe(fs.createWriteStream(`${profilePicDestination}${profilePicName}`));

                            await User.update({
                                profilePicture: profilePicName
                            }, {
                                where: {
                                    id: log.userId
                                }
                            });

                            await datingPageMediaService.createMedia({
                                userId: log.userId,
                                mediaUrl: fileName,
                                mediaType: 1,
                                isDeletable: 0,
                                mediaTypeText: MediaTypes.IMAGE
                            });
                        }

                        await IdenfyLogs.update({ succeed: 1, verificationFailed: 0 }, {
                            where: {
                                authToken: log.authToken
                            }
                        });

                    } else if ([IdenfyStatus.DENIED, IdenfyStatus.EXPIRED, IdenfyStatus.SUSPECTED].indexOf(documentStatus.status) > -1) {
                        await IdenfyLogs.update({ succeed: 1, verificationFailed: 1 }, {
                            where: {
                                authToken: log.authToken
                            }
                        });
                    }
                }
            } catch (error) {
                Logging.logMessage(error.message);
            }
        });

    } catch (e) {
        Logging.logGeneralError(e);
    }
});

// Cron at every 5 mins...
const sexOffenderScreeningCheckCron = new CronJob('*/5 * * * *', async () => {
    try {
        const unVerifiedProfiles = await SignUpCheckList.findAll({
            where: {
                documentVerified: 1,
                sexOffenderVerified: 0
            },
            raw: true
        });

        Async.each(unVerifiedProfiles, async (profile: any) => {
            try {
                const userProfile: any = await User.findOne({
                    where: {
                        id: profile.userId
                    }, raw: true
                });

                if (userProfile && userProfile.firstName && userProfile.lastName && userProfile.DOB) {
                    try {
                        const sexOffenderResult: any = await sexOffenderScreening({
                            firstName: userProfile.firstName,
                            lastName: userProfile.lastName,
                            DOB: userProfile.DOB
                        });

                        if (sexOffenderResult && sexOffenderResult.length > 0 && sexOffenderResult[0].success) {
                            if (sexOffenderResult[0].profiles.length > 0) {
                                await User.update({ isBlocked: 1 }, {
                                    where: {
                                        id: profile.userId
                                    }
                                });
                            }

                            await SignUpCheckList.update({ sexOffenderVerified: 1 }, {
                                where: {
                                    userId: profile.userId
                                }
                            });
                        }
                    } catch (error) {
                        Logging.logMessage(`Screening failed.. :${userProfile.id} due to Error: ${error}`)
                    }
                }

            } catch (error) {
                Logging.logMessage(error.message);
            }
        });

    } catch (e) {
        Logging.logGeneralError(e);
    }
});

export {
    storyCron,
    alertSystemCron,
    dvsVerificationCheckCron,
    idenfyVerificationCheckCron,
    sexOffenderScreeningCheckCron,
    datePageViewCountCron
};
