import request from 'request-promise';
import { DocumentTypes, HTTPStatusCode, IdenfyStatus, UserType, Gender } from "../../utills/enums";
import { Logging } from './logging.service';
import DocumentVerificationService from "../routes/documentVerification/documentVerification.service";
import UserService from "../routes/users/user.service";

const verifyDVS1 = async (data) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.DVS_VERIFY_URL}/Verify`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Bearer " + process.env.DVS_SECRET_KEY
        },
        json: true,
        body: {
            "documentType": data.documentType,
            "verifyFace": true,
            "frontImageBase64": data.frontImage.toString('base64'),
            "faceImageBase64": data.faceImage.toString('base64'),
            "overriddenSettings": {
                "isOCREnabled": true,
                "isAddressCheckEnabled": true,
                "isCrossMatchRequired": true,
                "isDMVEnabled": true,
                "isDMVPartialEnabled": true,
                "isIdentiFraudEnabled": true,
                "isIdentiFraudFullCaseEnabled": true,
                "isDmvOrIdentiFraudEnabled": true,
                "isDmvAndIdentiFraudEnabled": true,
                "isDmvOrIdentiFraudPartialEnabled": true,
                "isDmvAndIdentiFraudPartialEnabled": true,
                "isFaceRequiredOnDocument": true
            }
        }
    }
    if (data.backImage !== null && data.backImage !== "") options.body['backOrSecondImageBase64'] = data.backImage.toString('base64');

    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`DVS Verify Err... ${err}`);
            reject(err);
        });
});

const verifyFaceVideo = async (face, video) => new Promise ((resolve, reject) => {
    const sdk = require('api')(process.env.IDANALYSER_V2_PARAM);
    sdk.auth(process.env.IDANALYSER_KEY);
    let payload = {
        profile: 'security_medium'
    };
    payload['face'] = face.toString('base64');
    payload['faceVideo'] = video.toString('base64');
    return sdk.postLiveness(payload)
    .then((data) => resolve(data) )
    .catch((err) => {
        Logging.logMessage(`Video Verify Err... ${err}`);
        reject(err);
    });
});

const verifyDVS = async (user, data) => new Promise((resolve, reject) => {
    let payload = {'apikey': process.env.IDANALYSER_KEY};
    let faceImage;
    data.files && data.files.forEach( (file) => {
        if( file.fieldname === 'frontImage' ) {
            payload['file_base64'] = file.buffer.toString('base64');
        } else if (file.fieldname === 'backImage') {
            payload['file_back_base64'] = file.buffer.toString('base64');
        } else if (file.fieldname === 'faceImage') {
            payload['face_base64'] = file.buffer.toString('base64');
            faceImage = file;
        } else if (file.fieldname === 'video') {
            payload['video_base64'] = file.buffer.toString('base64');
        }
    });
    payload['authenticate'] = true;
    payload['authenticate_module'] = 2;
    request.post({
        uri: process.env.IDANALYSER_URL,
        headers: {'Content-Type': 'application/json'},
        json: true,
        body: payload,
    })
    .then(async (dvsResult) => {
        console.log('DVS Verification Res : ', dvsResult);
        let logData: any = {};
        let userDetailsToUpdate: any = {}
        if( dvsResult['error'] || !dvsResult['face']['isIdentical'] || !dvsResult['verification']['passed'] || dvsResult['authentication']['score'] < 0.5) {
            logData = {
                userId: user.id,
                verificaitionService: process.env.DOC_VERIFY_SERVICE,
                documentType: Object.keys(DocumentTypes)[Object.values(DocumentTypes).indexOf(data.body.documentType)],
                verificationId: dvsResult['responseID'],
                facePhotoSubmited: data.body.faceImage !== null && data.body.faceImage !== "",
                frontImageSubmited: data.body.frontImage !== null && data.body.frontImage !== "",
                backImageSubmited: data.body.backImage !== null && data.body.backImage !== "",
                succeeded: false
            };
            userDetailsToUpdate = {
                isDocumentVerified: 2
            }
        } else {
            logData = {
                userId: user.id,
                verificaitionService: process.env.DOC_VERIFY_SERVICE,
                documentType: Object.keys(DocumentTypes)[Object.values(DocumentTypes).indexOf(data.body.documentType)],
                verificationId: dvsResult['responseID'],
                facePhotoSubmited: data.body.faceImage !== null && data.body.faceImage !== "",
                frontImageSubmited: data.body.frontImage !== null && data.body.frontImage !== "",
                backImageSubmited: data.body.backImage !== null && data.body.backImage !== "",
                succeeded: true
            };
            userDetailsToUpdate = {
                firstName: dvsResult["result"]["firstName"],
                lastName: dvsResult["result"]["lastName"],
                DOB: dvsResult["result"]["dob"],
                birthYear: parseInt( dvsResult["result"]["dob"].slice(0, 4) ),
                gender: Gender[dvsResult['result']['sex']] || 'Unspecified',
                height: dvsResult["result"]["height"] || "",
                isDocumentVerified: 3
            }
            await DocumentVerificationService.createDatingMedia(faceImage, user.id);
        }
        await UserService.updateUser(user.id, userDetailsToUpdate);
        await DocumentVerificationService.createDocumentLogs(logData);
    })
    .catch(async (err) => {
        Logging.logMessage(`DVS Verify Err... ${err}`);
        await UserService.updateUser(user.id, { isDocumentVerified: 2 });
        await DocumentVerificationService.createDocumentLogs({
            userId: user.id,
            verificaitionService: process.env.DOC_VERIFY_SERVICE,
            documentType: Object.keys(DocumentTypes)[Object.values(DocumentTypes).indexOf(data.body.documentType)],
            verificationId: err['responseID'],
            facePhotoSubmited: data.body.faceImage !== null && data.body.faceImage !== "",
            frontImageSubmited: data.body.frontImage !== null && data.body.frontImage !== "",
            backImageSubmited: data.body.backImage !== null && data.body.backImage !== "",
            succeeded: false
        });
    });
});

const checkVerificationDVS = async (requestId) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.DVS_VERIFY_URL}/Request/${requestId}/result`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Bearer " + process.env.DVS_SECRET_KEY
        },
        json: true,
    }

    return request.get(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`DVS Check Verification Err... ${err}`);
            reject(err);
        });
});

const checkFaceConfidanceDVS = async (data) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.DVS_VERIFY_URL}/Analyze/face`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Bearer " + process.env.DVS_SECRET_KEY
        },
        json: true,
        body: {
            "faceImageBase64": data.faceImage.toString('base64')
        }
    };

    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`checkFaceConfidanceDVS Err... ${err}`);
            reject(err);
        });
}); 

const sexOffenderScreening = async (data) => new Promise((resolve, reject) => {
    const options = {
        uri: `${process.env.SCREENING_VERIFY_URL}/Check`,
        headers: {
            'Content-Type': 'application/json',
            Authorization: "Bearer " + process.env.SCREENING_VERIFY_TOKEN
        },
        json: true,
        body: {
            "firstName": data.firstName,
            "lastName": data.lastName,
            "dateOfBirths": data.DOB,
            "services": [process.env.SCREENING_VERIFY_SERVICEID]
        }
    };

    return request.post(options)
        .then((data) => resolve(data))
        .catch((err) => {
            Logging.logMessage(`sexOffenderScreening Err... ${err}`);
            reject(err);
        });
});

export {
    verifyDVS,
    verifyDVS1,
    checkVerificationDVS,
    checkFaceConfidanceDVS,
    sexOffenderScreening,
    verifyFaceVideo
}
