import request from 'request-promise';
import { ValidationModes } from '../../utills/enums';
import { Logging } from './logging.service';
const requestUrl = process.env.PG_LIVE_MODE == "true" ? process.env.PG_LIVE_URL : process.env.PG_SANDBOX_URL;
const validationMode = process.env.PG_LIVE_MODE == "true" ? ValidationModes.LIVEMODE : ValidationModes.TESTMODE;
const merchantAuthentication = {
    name: process.env.PG_MERCHANT_NAME,
    transactionKey: process.env.PG_TXN_KEY
};

const createCustomerProfile = (data) => new Promise((resolve, reject) => {
    const options = {
        uri: requestUrl,
        headers: {
            'Content-Type': 'application/json',
        },
        json: true,
        body: {
            createCustomerProfileRequest: {
                merchantAuthentication: merchantAuthentication,
                profile: {
                    merchantCustomerId: "",
                    description: "Individual Customer with Datesafe",
                    email: data.email,
                    paymentProfiles: {
                        customerType: "individual",
                        billTo: {
                            firstName: data.firstName || "",
                            lastName: data.lastName || "",
                            address: "",
                            city: "",
                            state: "",
                            zip: "",
                            country: "",
                            phoneNumber: data.phone || ""
                        },
                        payment: {
                            creditCard: {
                                cardNumber: data.cardNumber,
                                expirationDate: data.cardExpiryDate,
                                cardCode: data.cvv
                            }
                        }
                    }
                },
                validationMode: validationMode
            }
        }
    }

    return request.post(options)
        .then((data) => resolve(JSON.parse(data.trim())))
        .catch((err) => {
            Logging.logMessage(`PG Create Customer Err... ${err}`);
            reject(err);
        });
});

const createCustomerPaymentProfile = (data) => new Promise((resolve, reject) => {
    const options = {
        uri: requestUrl,
        headers: {
            'Content-Type': 'application/json',
        },
        json: true,
        body: {
            createCustomerPaymentProfileRequest: {
                merchantAuthentication: merchantAuthentication,
                customerProfileId: data.pgProfileId,
                paymentProfile: {
                    billTo: {
                        firstName: data.firstName || "",
                        lastName: data.lastName || "",
                        address: "",
                        city: "",
                        state: "",
                        zip: "",
                        country: "",
                        phoneNumber: data.phone || ""
                    },
                    payment: {
                        creditCard: {
                            cardNumber: data.cardNumber,
                            expirationDate: data.cardExpiryDate,
                            cardCode: data.cvv
                        }
                    },
                    defaultPaymentProfile: true
                },
                validationMode: validationMode
            }
        }
    };

    return request.post(options)
        .then((data) => resolve(JSON.parse(data.trim())))
        .catch((err) => {
            Logging.logMessage(`PG Create payment Profile Err... ${err}`);
            reject(err);
        });
});

const getCustomerProfile = (customerProfileId) => new Promise((resolve, reject) => {
    const options = {
        uri: requestUrl,
        headers: {
            'Content-Type': 'application/json',
        },
        json: true,
        body: {
            getCustomerProfileRequest: {
                merchantAuthentication: merchantAuthentication,
                customerProfileId: customerProfileId,
                unmaskExpirationDate: "true",
                includeIssuerInfo: "false"
            }
        }
    };

    return request.post(options)
        .then((data) => resolve(JSON.parse(data.trim())))
        .catch((err) => {
            Logging.logMessage(`PG Get customer Profile Err... ${err}`);
            reject(err);
        });
});

const chargeCustomerProfile = (customerProfileId, paymentProfileId, amount, cvv) => new Promise((resolve, reject) => {
    const options = {
        uri: requestUrl,
        headers: {
            'Content-Type': 'application/json',
        },
        json: true,
        body: {
            createTransactionRequest: {
                merchantAuthentication: merchantAuthentication,
                refId: Math.random().toString(36).slice(2),
                transactionRequest: {
                    transactionType: "authCaptureTransaction",
                    amount: amount,
                    currencyCode: process.env.PG_TXN_CURRENCY,  // USD
                    profile: {
                        customerProfileId: customerProfileId,
                        paymentProfile: {
                            paymentProfileId: paymentProfileId,
                            cardCode: cvv
                        }
                    },
                    processingOptions: {
                        isSubsequentAuth: "true"
                    },

                    authorizationIndicatorType: {
                        authorizationIndicator: "final"
                    }
                }
            }
        }
    };

    return request.post(options)
        .then((data) => resolve(JSON.parse(data.trim())))
        .catch((err) => {
            Logging.logMessage(`PG Charge customer profile Err... ${err}`);
            reject(err);
        });
});

const deleteCustomerPaymentProfile = (customerProfileId, paymentProfileId) => new Promise((resolve, reject) => {
    const options = {
        uri: requestUrl,
        headers: {
            'Content-Type': 'application/json',
        },
        json: true,
        body: {
            deleteCustomerPaymentProfileRequest: {
                merchantAuthentication: merchantAuthentication,
                customerProfileId: customerProfileId,
                customerPaymentProfileId: paymentProfileId
            }
        }
    };

    return request.post(options)
        .then((data) => resolve(JSON.parse(data.trim())))
        .catch((err) => {
            Logging.logMessage(`PG delete pay profile Err... ${err}`);
            reject(err);
        });
});

export {
    createCustomerProfile,
    createCustomerPaymentProfile,
    getCustomerProfile,
    chargeCustomerProfile,
    deleteCustomerPaymentProfile
}