import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import BlockUserService from "./blockUser.service";
import UserService from "../users/user.service";
import { CustomError } from "../../../utills/types";
import FollowingFollowerService from "../followingFollower/followingFollower.service";

const addBlockUser = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
        const blockInsertData = {
            blockByUserId: user.id,
            blockForUserId: req.body.userId
        };

        await BlockUserService.createBlockUser(blockInsertData);

        //delete follower followings
        await FollowingFollowerService.removeFollowRequest(blockInsertData.blockByUserId, blockInsertData.blockForUserId);
        await FollowingFollowerService.removeFollowRequest(blockInsertData.blockForUserId, blockInsertData.blockByUserId);
        

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getCreatedMessage('Block User')
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: StaticMessages.USER_BLOCK_FAILED
        });
    }
};

const getBlockUser = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }

        const blockedUsers = await BlockUserService.getBlockUserByUserId(user.id);

        if (blockedUsers.length == 0) {
            throw new CustomError(StaticMessages.getNotExistMessage('Block User'), HTTPStatusCode.NOTFOUND);
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Blocked User'),
            blockedUsers
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: getErrorMessage(error).message
        });
    }
};

const unblockUser = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }

        const unblock = await BlockUserService.deleteBlockUserByUserId(user.id, req.body.userId);

        if (unblock !== 0) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Unblock User'),
                unBlockUsers: unblock
            });
        } else {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Unblock User'),
                unBlockUsers: unblock
            });
        }

    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: getErrorMessage(error).message
        });
    }
};

export default {
    addBlockUser,
    getBlockUser,
    unblockUser
}