import { Request, Response } from "express";
import { DatePageActions, FollowStatus, HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import DatePageActionService from "./datePageAction.service";
import UserService from "../users/user.service";
import moment from "moment";
import getErrorObject from "../../../utills/errorStructure";
import { Logging } from "../../commonServices/logging.service";
import { CustomError } from "../../../utills/types";
import FollowingFollowerService from "../followingFollower/followingFollower.service";
import SubscribedUserService from "../subscribedUser/subscribedUser.service";

const addNewAction = async (req: Request, res: Response) => {
    const user = req['user'];
    const checkUserExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(user.id)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('User'),
                    statusCode: HTTPStatusCode.NOTFOUND,
                    name: Constants.CUSTOM_ERROR
                });
                resolve();
            }).catch((err) => reject(err))
    });

    const checkActionUserExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(req.body.actionUserId)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('Action User'),
                    statusCode: HTTPStatusCode.NOTFOUND,
                    name: Constants.CUSTOM_ERROR
                });
                resolve();
            }).catch((err) => reject(err))
    });

    const checkUniqueActions = () => new Promise<void>((resolve, reject) => {
        return DatePageActionService.getAllActionByUserIdActionUserId(req.body.actionUserId, user.id)
            .then((data) => {
                if (data.length > 0) {
                    return res.status(HTTPStatusCode.SUCCESS).json({
                        status: Constants.SUCCESS,
                        message: StaticMessages.getCreatedMessage('Dating Action')
                    });
                }
                resolve();
            }).catch((err) => reject(err))
    });

    const checkForUserDailyViewCount = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(user.id, ['datePageViewCount'])
            .then(async (data) => {
                const userSubscription = await SubscribedUserService.getActiveSubscriptionByUserId(user.id);
                if (userSubscription.length > 0) {
                    return resolve()
                } else {
                    if (data.datePageViewCount <= Constants.DATEPAGE_PROFILE_VISIT) {
                        return resolve()
                    } else {
                        return reject({
                            message: StaticMessages.DATE_PAGE_VIEW_COUNT_EXCEED,
                            statusCode: HTTPStatusCode.ERROR_412,
                            name: Constants.CUSTOM_ERROR
                        });
                    }
                }
            }).catch((err) => reject(err))
    });

    const makeMatchOfEachOther = () => new Promise<void>(async (resolve, reject) => {
        try {
            if (req.body.actionType == DatePageActions.LIKE) {
                const actionUserActionStatus = await DatePageActionService.getAllActionByUserIdActionUserIdActionType(user.id, req.body.actionUserId, DatePageActions.LIKE);
                if (actionUserActionStatus.length > 0) {
                    await FollowingFollowerService.followUserWithStatus(user.id, req.body.actionUserId, FollowStatus.FOLLOW_REQUEST_ACCEPTED);
                    await FollowingFollowerService.followUserWithStatus(req.body.actionUserId, user.id, FollowStatus.FOLLOW_REQUEST_ACCEPTED);
                }
            } else {
                await FollowingFollowerService.removeFollowRequest(user.id, req.body.actionUserId);
                await FollowingFollowerService.removeFollowRequest(req.body.actionUserId, user.id);
            }

            return resolve();
        } catch (error) {
            return reject(error);
        }
    });

    const addDatingPageAction = () => new Promise<void>(async (resolve, reject) => {
        const datingPageActionData = {
            userId: user.id,
            actionUserId: req.body.actionUserId,
            actionType: req.body.actionType == DatePageActions.LIKE ? DatePageActions.LIKE : DatePageActions.IGNORE,
            timestamp: moment().unix()
        }

        await UserService.incrementDatePageViewCount(user.id);

        return DatePageActionService.createAction(datingPageActionData)
            .then(() => resolve())
            .catch(err => reject(err));
    });

    return checkUserExistOrNot()
        .then(checkActionUserExistOrNot)
        .then(checkUniqueActions)
        // .then(checkForUserDailyViewCount)
        .then(makeMatchOfEachOther)
        .then(addDatingPageAction)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Dating Action')
            });
        })
        .catch((err) => {
            Logging.logMessage(err)
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                err = getErrorObject(err)
                return res.status(HTTPStatusCode.SUCCESS).json({
                    error: err,
                    message: err.message,
                    status: Constants.ERROR
                });
            };
        });
};

const getDatePageLikes = async (req: Request, res: Response) => {
    const user = req['user'];
    const checkUserExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(user.id)
            .then((data) => {
                if (!data) throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
                resolve();
            }).catch((err) => reject(err))
    });

    const getLikeWithUserInfo = () => new Promise((resolve, reject) => {
        return DatePageActionService.getActionsByUserIdWithUserInfo(user.id)
            .then((data) => {
                if (data.length > 0) {
                    resolve(data);
                } else {
                    throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
                }
            }).catch((err) => reject(err))
    });

    return checkUserExistOrNot()
        .then(getLikeWithUserInfo)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Dating Action'),
                users: data
            });
        })
        .catch((err) => {
            Logging.logMessage(err)
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                err = getErrorObject(err)
                return res.status(HTTPStatusCode.SUCCESS).json({
                    error: err,
                    message: err.message,
                    status: Constants.ERROR
                });
            };
        });
};

const deleteAction = async (req: Request, res: Response) => {
    const user = req['user'];
    const checkUserExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(user.id)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('User'),
                    statusCode: HTTPStatusCode.NOTFOUND,
                    name: Constants.CUSTOM_ERROR
                });
                resolve();
            }).catch((err) => reject(err))
    });

    const checkActionExistsOrNot = () => new Promise<void>((resolve, reject) => {
        return DatePageActionService.getAllActionByUserIdActionUserId(req.body.actionUserId, user.id)
            .then((data) => {
                if (data.length > 0) {
                    resolve()
                } else {
                    reject({
                        message: StaticMessages.getNotExistMessage('Action'),
                        statusCode: HTTPStatusCode.NOTFOUND,
                        name: Constants.CUSTOM_ERROR
                    });
                }
            }).catch((err) => reject(err))
    });

    const deleteDatingPageAction = () => new Promise<void>((resolve, reject) => {
        return DatePageActionService.deleteActionByUserIdActionUserId(req.body.actionUserId, user.id)
            .then(() => resolve())
            .catch(err => reject(err));
    });

    return checkUserExistOrNot()
        .then(checkActionExistsOrNot)
        .then(deleteDatingPageAction)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getDeletedMessage('Dating Action')
            });
        }).catch((err) => {
            Logging.logMessage(err)
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                err = getErrorObject(err)
                return res.status(HTTPStatusCode.SUCCESS).json({
                    error: err,
                    message: err.message,
                    status: Constants.ERROR
                });
            };
        });
};

export default {
    addNewAction,
    deleteAction,
    getDatePageLikes
};