import { Request, Response } from "express";
import { FolderNames, HTTPStatusCode, ImageExentions, MediaSources, MediaTypes, VideoExtensions } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import DatingPageMediaService from "./datingPageMedia.service";
import UserService from "../users/user.service";
import path from 'path';
import { handleFileDelete, handleFileUpload } from "../../commonServices/fsFileUpload";
import { CustomError } from "../../../utills/types";
import getErrorObject from "../../../utills/errorStructure";
import signUpCheckListService from "../signUpCheckList/signUpCheckList.service";
import DocumentVerificationService from "../documentVerification/documentVerification.service";

const addMedias = async (req: Request, res: Response) => {
    const user = req['user'];

    const checkUserExistOrNot = () => new Promise((resolve, reject) => {
        return UserService.getUserById(user.id)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('User'),
                    status: HTTPStatusCode.NOTFOUND
                });
                resolve(data);
            }).catch((err) => reject(err))
    });

    const uploadMediaAndMakeEntryInDb = () => new Promise<void>(async (resolve, reject) => {
        try {
            let uploadedFilePaths;
            if (req.body.mediaPath.length > Constants.DATEPAGE_MEDIA_ALLOW_LIMIT) throw new CustomError(StaticMessages.DATING_PAGE_MEDIA_NOT_ALLOWED_MORE, HTTPStatusCode.BAD_REQUEST);

            uploadedFilePaths = await handleFileUpload({
                type: MediaSources.DATING_PAGE_MEDIA,
                userId: user.id,
                itemId: '0',
                files: req.body.mediaPath
            });
            uploadedFilePaths.map(async (filePath) => {
                let pathExt = path.extname(filePath);
                if (Object.values(ImageExentions).findIndex((imgEx) => imgEx == pathExt) > -1) {
                    await DatingPageMediaService.createMedia({
                        userId: user.id,
                        mediaUrl: filePath,
                        mediaType: 1,
                        isDeletable: 1,
                        mediaTypeText: MediaTypes.IMAGE,
                        isPrimaryImage: req.body.isPrimaryImage
                    });

                    if (req.body.isPrimaryImage === "1") {
                        await DocumentVerificationService.createDatingMedia(req.body.mediaPath[0], user.id, false);
                    }
                } else if (Object.values(VideoExtensions).findIndex((vidEx) => vidEx == pathExt) > -1) {
                    await DatingPageMediaService.createMedia({
                        userId: user.id,
                        mediaUrl: filePath,
                        mediaType: 2,
                        isDeletable: 1,
                        mediaTypeText: MediaTypes.VIDEO,
                        isPrimaryImage: req.body.isPrimaryImage
                    });
                };
            });

            await signUpCheckListService.updateCheckListByUserId(user.id, { datingPageMediaUploaded: true });
            return resolve();
        } catch (err) {
            return reject(err);
        }
    });

    return checkUserExistOrNot()
        .then(uploadMediaAndMakeEntryInDb)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Dating Medias'),
            });
        })
        .catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                err = getErrorObject(err);
                return res.status(HTTPStatusCode.SUCCESS).json({
                    error: err,
                    status: Constants.ERROR,
                    message: err.message
                });
            };
        })
};

const getMedias = async (req: Request, res: Response) => {
    const user = req['user'];
    const checkUserExistOrNot = () => new Promise((resolve, reject) => {
        return UserService.getUserById(user.id)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('User'),
                    status: HTTPStatusCode.NOTFOUND
                });
                resolve(data);
            }).catch((err) => reject(err));
    });

    const getMediaList = () => new Promise((resolve, reject) => {
        return DatingPageMediaService.getMediaByUserIdWithUserDetails(user.id)
            .then((oldMedia) => {
                oldMedia.map(async (media) => {
                    media.mediaUrl = '/static/users/' + media.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/' + media.mediaUrl;
                    delete media.User;
                    delete media['User.userName'];
                    delete media['User.lastName'];
                    delete media['User.firstName'];
                });
                resolve(oldMedia);
            }).catch((err) => reject(err));
    });

    return checkUserExistOrNot()
        .then(getMediaList)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Dating Medias'),
                medias: data
            });
        })
        .catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: err.status,
                    message: err.message
                });
            }
        });
};

const deleteMedias = async (req: Request, res: Response) => {
    const user = req['user'];
    const checkUserExistOrNot = () => new Promise((resolve, reject) => {
        return UserService.getUserById(user.id)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('User'),
                    status: HTTPStatusCode.NOTFOUND
                });
                resolve(data);
            }).catch((err) => reject(err));
    });

    const checkMediaExistOrNot = () => new Promise((resolve, reject) => {
        return DatingPageMediaService.getSingleMediaByIdAndUserId(req.body.id, user.id)
            .then((data) => {
                if (!data) reject({
                    message: StaticMessages.getNotExistMessage('Dating Media'),
                    status: HTTPStatusCode.NOTFOUND
                });
                resolve(data);
            }).catch((err) => reject(err));
    });

    const deleteMedia = (media) => new Promise<void>((resolve, reject) => {
        return DatingPageMediaService.deleteMediaByMediaId(media.id)
            .then(async () => {
                let deleteHandler = {
                    type: MediaSources.DATING_PAGE_MEDIA,
                    userId: user.id,
                    itemId: '0',
                    filePath: media.mediaUrl
                };
                await handleFileDelete(deleteHandler);
                resolve();
            }).catch((err) => reject(err));
    });

    return checkUserExistOrNot()
        .then(checkMediaExistOrNot)
        .then(deleteMedia)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getDeletedMessage('Dating Media')
            });
        })
        .catch((err) => {
            Logging.logMessage(err);
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: err.status,
                    message: err.message
                });
            }
        });
};
export default {
    addMedias,
    getMedias,
    deleteMedias
};
