import { Request, Response } from "express";
import { DocumentTypes, HTTPStatusCode, IdenfyStatus, UserType, Gender } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import UserService from "../users/user.service";
import Async from "async";
import { verifyDVS, verifyFaceVideo, verifyDVS1, checkFaceConfidanceDVS, checkVerificationDVS } from "../../commonServices/dvsService";
import { CustomError } from "../../../utills/types";
import DocumentVerificationService from "./documentVerification.service";
import { deleteIdenfyData, getIdenfyData, getIdenfyFiles, getIdenfyStatus, getIdenfyToken } from "../../commonServices/idenfyService";
import SignUpCheckListService from "../signUpCheckList/signUpCheckList.service";
import getErrorObject from "../../../utills/errorStructure";

const verifyDocument1 = async (req: Request, res: Response) => {
    try {
        const user = await UserService.getVerifiedUserByEmail(req.body.email);
        if (!user) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        } else if (!user.isVerified && user.userType == UserType.USER) {
            throw new CustomError('Email is Not Verified.', HTTPStatusCode.BAD_REQUEST);
        }
        const faceQuelityCheck = await checkFaceConfidanceDVS(req.body);
        if (faceQuelityCheck && typeof faceQuelityCheck['realfaceConf'] !== "undefined" && faceQuelityCheck['realfaceConf'] < Constants.DEFAULT_FACE_CONFIDANCE) {
            throw new CustomError(StaticMessages.getBadQualityPhoto('Face Photo'), HTTPStatusCode.BAD_REQUEST);
        };

        const dvsResult = await verifyDVS1(req.body);
        if (dvsResult && dvsResult['requestId']) {
            const logData = {
                userId: user.id,
                verificaitionService: process.env.DOC_VERIFY_SERVICE,
                documentType: Object.keys(DocumentTypes)[Object.values(DocumentTypes).indexOf(req.body.documentType)],
                verificationId: dvsResult['requestId'],
                facePhotoSubmited: req.body.facePhoto !== null && req.body.facePhoto !== "",
                frontImageSubmited: req.body.frontImage !== null && req.body.frontImage !== "",
                backImageSubmited: req.body.backImage !== null && req.body.backImage !== "",
                succeeded: false
            };
            await DocumentVerificationService.createDocumentLogs(logData);
            await UserService.updateUser(user.id, {
                firstName: dvsResult["document"]["firstName"],
                lastName: dvsResult["document"]["familyName"],
                DOB: dvsResult["document"]["dob"]
            });
        } else {
            throw new CustomError(StaticMessages.DOCUMENT_ERROR, 400)
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.DOCUMENT_VERIFICATION_INIT,
            requestId: dvsResult['requestId']
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
};

const verifyVideo = async (req: Request, res: Response) => {
    try {
        let authUser = req['user'];
        let videoVerificationResults: any = await verifyFaceVideo(req.body.faceImage ,req.body.video);
        console.log('video Verification Res : ', videoVerificationResults);
        if (!videoVerificationResults['data'] ||
            !videoVerificationResults['data']['decision'] ||
            videoVerificationResults['data']['decision'] !== 'accept') {
                throw new CustomError(StaticMessages.VIDEO_VERIFICATION_FAILED, 400);
        }

        await DocumentVerificationService.setVideoVerificationStatus(authUser.id);
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.VIDEO_VERIFICATION_PASSED,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
};

const verifyDocument = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];
        const user = await UserService.getUserById(authUser.id);
        if (!user) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        } else if (!user.isVerified && user.userType == UserType.USER) {
            throw new CustomError('Email is Not Verified.', HTTPStatusCode.BAD_REQUEST);
        }

        await UserService.updateUser(authUser.id, {isDocumentVerified: 1});
        verifyDVS(authUser, req);

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.DOCUMENT_VERIFICATION_INIT
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
};

const checkStatus = async (req: Request, res: Response) => {
    try {
        const requestId = String(req.query.requestId);

        const requestData = await DocumentVerificationService.getLogsByRequestId(requestId);
        if (!requestData)
            throw new CustomError(StaticMessages.getNotExistMessage('Request'), HTTPStatusCode.BAD_REQUEST);

        if (requestData.succeeded == 0 && requestData.verificationFailed == 0) {
            const checkedData = await checkVerificationDVS(requestId);
            if (checkedData && checkedData["request"]["status"] == 1) {
                if (checkedData['verificationResult']['faceVerificationResult']["confidence"] > Constants.DEFAULT_FACE_CONFIDANCE) {
                    await UserService.updateDocumentVerification(requestData.userId);
                    await DocumentVerificationService.updateLogsByRequestId({ succeeded: 1 }, requestId);
                } else {
                    await DocumentVerificationService.updateLogsByRequestId({ succeeded: 1, verificationFailed: 1 }, requestId);
                    throw new CustomError(StaticMessages.DOCUMENT_FAILED, HTTPStatusCode.CONFLICT);
                }
            }
        } else if (requestData.succeeded == 1 && requestData.verificationFailed == 1) {
            throw new CustomError(StaticMessages.DOCUMENT_FAILED, HTTPStatusCode.CONFLICT);
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.DOCUMENT_VERIFIED,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.DOCUMENT_FAILED,
            status: Constants.ERROR
        });
    }
};

const getToken = async (req: Request, res: Response) => {
    try {
        const idenfyToken: any = await getIdenfyToken();
        if (idenfyToken && idenfyToken.authToken) {
            await DocumentVerificationService.createIdenfyLogs({
                userId: req['user'].id,
                authToken: idenfyToken.authToken,
                clientId: idenfyToken.clientId,
                scanRef: idenfyToken.scanRef,
                succeed: false,
                verificationFailed: false,
                verificaitionService: process.env.IDENFY_VERIFY_SERVICE,
                verificationStatus: IdenfyStatus.ACTIVE
            })
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Token'),
                authToken: idenfyToken.authToken
            });
        } else {
            throw new CustomError(StaticMessages.getNotCreatedMessage('Token'), HTTPStatusCode.BAD_REQUEST);
        }

    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
}

const getStatus = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];
        // const getUserIdenfyLogs = await DocumentVerificationService.getIdenfyLogsByUserIdWithIdSort(authUser.id);
        // if (getUserIdenfyLogs && getUserIdenfyLogs.length > 0) {

        //     const idenfyStatus: any = await getIdenfyStatus(getUserIdenfyLogs[0].scanRef)
        //     // temporarily not validating for the testing purpose
        //     // if (idenfyStatus.status == IdenfyStatus.APPROVED) {
        //     // }
        //     await SignUpCheckListService.updateCheckListByUserId(authUser.id, { documentVerified: true });
        //     await UserService.updateDocumentVerification(authUser.id);
        //     return res.status(HTTPStatusCode.SUCCESS).json({
        //         status: Constants.SUCCESS,
        //         message: StaticMessages.getItemMessage('Status Found'),
        //         verificationStatus: idenfyStatus.status
        //     });
        const result = await DocumentVerificationService.getVerificationStatus(authUser.id);
        if(result === null) throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getItemMessage('Status Found'),
            verificationStatus: result
        });
        // } else {
        //     throw new CustomError(StaticMessages.getNotExistMessage('Verification Requests'), HTTPStatusCode.NOTFOUND);
        // }

    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
}

const getData = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];

        const getUserIdenfyLogs = await DocumentVerificationService.getIdenfyLogsByUserIdWithIdSort(authUser.id);
        if (getUserIdenfyLogs && getUserIdenfyLogs.length > 0) {

            try {
                const idenfyData: any = await getIdenfyData(getUserIdenfyLogs[0].scanRef);

                if (!idenfyData.docFirstName) {
                    throw new CustomError(StaticMessages.getNotExistMessage('Data'), HTTPStatusCode.NOTFOUND);
                }
                const verificationData = {
                    firstName: idenfyData && idenfyData.docFirstName,
                    lastName: idenfyData && idenfyData.docLastName
                }

                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.SUCCESS,
                    message: StaticMessages.getItemMessage('Data Found'),
                    verificationData: verificationData
                });
            } catch (error) {
                throw new CustomError(StaticMessages.getNotExistMessage('Data'), HTTPStatusCode.NOTFOUND);
            }
        } else {
            throw new CustomError(StaticMessages.getNotExistMessage('Verification Requests'), HTTPStatusCode.NOTFOUND);
        }

    } catch (err: any) {
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            err = getErrorObject(err)
            return res.status(HTTPStatusCode.SUCCESS).json({
                error: err,
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
}

const getFiles = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];

        const getUserIdenfyLogs = await DocumentVerificationService.getIdenfyLogsByUserIdWithIdSort(authUser.id);
        if (getUserIdenfyLogs && getUserIdenfyLogs.length > 0) {

            const idenfyFiles: any = await getIdenfyFiles(getUserIdenfyLogs[0].scanRef)
            if (idenfyFiles && idenfyFiles.fileUrls && Object.keys(idenfyFiles.fileUrls).length > 0) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.SUCCESS,
                    message: StaticMessages.getItemMessage('Document Files Found'),
                    verificationStatus: idenfyFiles.fileUrls
                });
            } else {
                throw new CustomError(StaticMessages.getNotExistMessage('Verification Files'), HTTPStatusCode.NOTFOUND);
            }

        } else {
            throw new CustomError(StaticMessages.getNotExistMessage('Verification Requests'), HTTPStatusCode.NOTFOUND);
        }

    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
}

const deleteData = async (req: Request, res: Response) => {
    try {

        const idenfyData = await DocumentVerificationService.getIdenfyLogs({
            succeeded: 0
        });

        Async.each(idenfyData, async (idenfyLog: any) => {
            await deleteIdenfyData(idenfyLog.scanRef);

            await DocumentVerificationService.deleteIdenfyLogsByScanRef(idenfyLog.scanRef);
        });

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getItemMessage('Data Deleted'),
        });

    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.message,
            status: Constants.ERROR
        });
    }
}

export default {
    verifyDocument,
    verifyVideo,
    checkStatus,
    getToken,
    getData,
    getStatus,
    getFiles,
    deleteData,
    verifyDocument1
};
