import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";


const verifyDocValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Document Verification Validator  -> verifyDocValidator")
    try {
        const schema = Joi.object({
            email: Joi.string().optional().allow('').description('Email'),
            documentType: Joi.number().valid(1, 2, 3, 6, 7).required().error(new Error('Document type is Missing or Invalid')),
            faceImage: Joi.any().allow('').description('faceImage').required().error(new Error('faceImage is missing')),
            video: Joi.any().allow('').description('video').required().error(new Error('video is missing')),
            frontImage: Joi.any().allow('').description('frontImage').required().error(new Error('frontImage is missing')),
            backImage: Joi.any().allow('').optional().default(null).description('backImage').error(new Error('backImage is missing'))
        });

        if (req['files']) {
            var fileKeys = Object.keys(req.files);
            fileKeys.forEach(function (key) {
                req.body[req.files[key]['fieldname']] = req.files[key]['buffer'];
            });
        }
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                error: {
                    message: validation.error.message,
                    statusCode: HTTPStatusCode.BAD_REQUEST
                },
                message: validation.error.message,
                status: Constants.ERROR
            });
        }

        req.body = validation.value;
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const verifyVideoValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Document Verification Validator  -> verifyDocValidator")
    try {
        const schema = Joi.object({
            faceImage: Joi.any().allow('').description('faceImage').required().error(new Error('faceImage is missing')),
            video: Joi.any().allow('').description('video').required().error(new Error('video is missing')),
        });

        if (req['files']) {
            var fileKeys = Object.keys(req.files);
            fileKeys.forEach(function (key) {
                req.body[req.files[key]['fieldname']] = req.files[key]['buffer'];
            });
        }
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                error: {
                    message: validation.error.message,
                    statusCode: HTTPStatusCode.BAD_REQUEST
                },
                message: validation.error.message,
                status: Constants.ERROR
            });
        }

        req.body = validation.value;
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const verificationCheckResultValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("Document Verification Validator  -> verificationCheckResultValidator")
    try {
        const schema = Joi.object({
            requestId: Joi.string().required().description('requestId').error(new Error('requestId is missing'))
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                error: {
                    message: validation.error.message,
                    statusCode: HTTPStatusCode.BAD_REQUEST
                },
                message: validation.error.message,
                status: Constants.ERROR
            });
        }

        req.query = validation.value;
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    verifyDocValidator,
    verificationCheckResultValidator,
    verifyVideoValidator
}