import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import AlertService from "./alert.service";
import { Logging } from "../../commonServices/logging.service";
import getErrorObject from "../../../utills/errorStructure";
import UserService from "../users/user.service";
import moment from "moment";
import { CustomError } from "../../../utills/types";

const addEmegencyContacts = async (req: Request, res: Response) => {
    try {
        let authUser = req['user'];
        await AlertService.removeEmergencyContacts(authUser.id);
        await UserService.updateUser(authUser.id, { alertSystemConsent: parseInt(req.body.alertSystemConsent) })
        let createResponse = await AlertService.addEmergencyContactsWithUserId(req.body.contacts, authUser.id);
        return res.status(HTTPStatusCode.SUCCESS).json({
            emergencyContacts: createResponse,
            message: StaticMessages.getCreatedMessage('Emergency Contact'),
            status: Constants.SUCCESS
        });
    } catch (err) {
        let error = getErrorObject(err);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getCreationFailedMessage('Emergency Contact'),
            status: Constants.ERROR
        });
    }
}

const getEmergencyContacts = async (req: Request, res: Response) => {
    try {
        let authUser = req['user'];
        const user = await UserService.getUserById(authUser.id, ["alertSystemConsent"]);
        let contacts = await AlertService.getEmergencyContacts(authUser.id);
        if (contacts.length > 0) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                emergencyContacts: { contacts, alertSystemConsent: user.alertSystemConsent },
                message: StaticMessages.getListReturnedMessage('Emergency Contact'),
                status: Constants.SUCCESS
            });
        } else {
            throw new CustomError(StaticMessages.getNotExistMessage('Emergency Contact'), HTTPStatusCode.NOTFOUND);
        }
    } catch (err) {
        let error = getErrorObject(err);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Emergency Contact'),
            status: Constants.ERROR
        });
    }
};

const setIsOnDate = async (req: Request, res: Response) => {
    try {
        let authUser = req['user'];
        await UserService.setIsOnDate(authUser.id, req.body);
        if (req.body.isOnDate) {
            await AlertService.addTimeLongs({
                userId: authUser.id,
                lastTimeStamp: moment().unix(),
                latitude: req.body.latitude,
                longitude: req.body.longitude,
            });
        }
        return res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.UPATE_DATE_STATUS,
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.OP_FAILED,
            status: Constants.ERROR
        });
    }
};

const addTimeLog = async (req: Request, res: Response) => {
    try {
        let authUser = req['user'];

        await AlertService.removeTimeLogsByUserId(authUser.id);
        await AlertService.addTimeLongs({
            userId: authUser.id,
            lastTimeStamp: moment().unix(),
            latitude: req.body.latitude,
            longitude: req.body.longitude,
        });
        return res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getCreatedMessage('Timelogs'),
            status: Constants.SUCCESS
        });

    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.OP_FAILED,
            status: Constants.ERROR
        });
    }
};
export default {
    addEmegencyContacts,
    setIsOnDate,
    getEmergencyContacts,
    addTimeLog
}