import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import FollowingFollowerService from "./followingFollower.service";
import UserService from "../users/user.service";
import { StaticMessages } from "../../../utills/staticMessage";
import { FollowStatus } from "../../../utills/enums";

const followUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let currentUserId = req['user'].id;
        if (await UserService.exists({
            id: req.body.toUser
        }) && await UserService.exists({
            id: currentUserId
        })) {
            let user = await FollowingFollowerService.followUser(req.body.toUser, currentUserId);
            res.status(HTTPStatusCode.CREATED).json({
                user: user,
                status: Constants.SUCCESS,
                message: StaticMessages.FOLLOW_REQUEST_SENT
            });
        } else {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getNotExistMessage('User')
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const getUserFollowings = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userId = req['user'].id;
        if (req.params.id) {
            userId = req.params.id;
        }
        let followings = await FollowingFollowerService.getUserFollowings(userId, parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
        if (followings) {
            res.status(HTTPStatusCode.SUCCESS).json({
                Followings: followings,
                message: StaticMessages.getItemMessage("Followings of user"),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const getUserFollowers = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userId = req['user'].id;
        if (req.params.id) {
            userId = req.params.id;
        }
        let followers = await FollowingFollowerService.getUserFollowers(userId, req.query.limit, req.query.offset);
        if (followers) {
            res.status(HTTPStatusCode.SUCCESS).json({
                followers: followers,
                message: StaticMessages.getItemMessage("Followers of user"),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const deleteUserFollowRequest = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let currentUserId = req['user'].id;
        let followers = await FollowingFollowerService.removeFollowRequest(req.body.toUser, currentUserId);
        if (followers < 1) {
            res.status(HTTPStatusCode.SUCCESS).json({
                followers: followers,
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                followers: followers,
                message: StaticMessages.FOLLOW_REQUEST_DELETED,
                status: Constants.SUCCESS
            })
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const acceptFollowRequest = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let currentUserId = req['user'].id;
        let status = req.body.status;
        let followers = await FollowingFollowerService.updateFollowRequest(currentUserId, req.body.fromUser, status);
        if (status === FollowStatus.FOLLOW_REQUEST_ACCEPTED) {
            res.status(HTTPStatusCode.SUCCESS).json({
                followers: followers,
                message: StaticMessages.FOLLOW_REQUEST_ACCEPTED,
                status: Constants.SUCCESS
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const UnFollowUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let currentUserId = req['user'].id;
        let followers = await FollowingFollowerService.removeFollowRequest(req.body.toUser, currentUserId);
        res.status(HTTPStatusCode.SUCCESS).json({
            followers: followers,
            message: StaticMessages.UNFOLLOW_USER,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const rejectUserFollowRequest = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let currentUserId = req['user'].id;
        let followers = await FollowingFollowerService.removeFollowRequest(currentUserId, req.body.fromUser);
        res.status(HTTPStatusCode.SUCCESS).json({
            followers: followers,
            message: StaticMessages.FOLLOW_REQUEST_REJECTED,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const followRequestList = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let currentUserId = req['user'].id;
        let requests = await FollowingFollowerService.getFollowRequestListByUserId(currentUserId);
        res.status(HTTPStatusCode.SUCCESS).json({
            requests: requests,
            message: StaticMessages.getListReturnedMessage('Follow request'),
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

export default {
    followUser,
    getUserFollowings,
    getUserFollowers,
    deleteUserFollowRequest,
    UnFollowUser,
    acceptFollowRequest,
    rejectUserFollowRequest,
    followRequestList
}