import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { HTTPStatusCode, OtpType } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import { StaticMessages } from "../../../utills/staticMessage";
import ForgotResetPasswordService from "./forgotResetPassword.service";
import UserService from "../users/user.service";
import OtpService from "../otp/otp.service";
import { CustomError } from "../../../utills/types";

const validateOTP = async (req: Request, res: Response, next: NextFunction) => {
    try {
        await OtpService.validateOtpOnSms(req.body.countryCode, req.body.phone, req.body.otp, OtpType.RESET_PASSWORD, 0);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.OTP_VERIFIED
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.OTP_NOT_VERIFIED,
            status: Constants.ERROR
        });
    }
};

const forgotPassword = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const userExists = await UserService.exists({ countryCode: req.body.countryCode, phone: req.body.phone });
        if (userExists) {
            await OtpService.sendOtpOnSms(req.body.countryCode, req.body.phone, OtpType.RESET_PASSWORD);
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.OTP_SENT + `${req.body.countryCode}${req.body.phone}`
            });
        } else {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }
    } catch (error: any) {
        error = getErrorMessage(error);
        Logging.logRequestError(req, error);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.OP_FAILED
        });
    }
};

const resetPassword = async (req: Request, res: Response, next: NextFunction) => {
    try {
        await ForgotResetPasswordService.resetPassword(req.body.countryCode, req.body.phone, req.body.password);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.PASSWORD_UPDATED
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.PASSWORD_NOT_UPDATED,
            status: Constants.ERROR
        });
    }
};

export default {
    forgotPassword,
    resetPassword,
    validateOTP
}