import { Request, Response, NextFunction } from "express";
import locationService from "./location.service";
import { HTTPStatusCode, locationPrivacy } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";

const createLocation = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        req.body.locationPrivacy = locationPrivacy.ANYONE;
        let previousLocation = await locationService.getLocationByUserId(user.id);
        if (previousLocation) {
            req.body.locationPrivacy = previousLocation.locationPrivacy;
        }

        await locationService.deleteLocationByUserId(user.id);
        let data = await locationService.createLocation(req.body, user.id);
        if (data) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Location'),
                user: data,
            });
        };
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.getCreationFailedMessage('Location')
        });
    }
};

const getLocationByUserId = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'].id;
        let users = await locationService.getLocationByUserId(authUser);
        if (users) {
            res.status(HTTPStatusCode.SUCCESS).json({
                users: users,
                message: StaticMessages.getItemMessage("Location of User."),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.NO_RECORD_FOUND,
                statusCode: HTTPStatusCode.SUCCESS
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
}

const updateLocationPrivacy = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'].id;
        let users = await locationService.updateLocationByUserId(authUser, req.body);
        if (users.length > 0 && users[0] != 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                users: users,
                message: StaticMessages.getUpdatedMessage("Location of User"),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.ERROR
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.getUpdateFailedMessage('location')
        });
    }
};

const deleteLocationByUserId = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'].id;
        let users = await locationService.deleteLocationByUserId(authUser);
        if (users) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getDeletedMessage("Location of User"),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.ERROR
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.getDeletedFailedMessage('location')
        });
    }
};

export default {
    createLocation,
    getLocationByUserId,
    updateLocationPrivacy,
    deleteLocationByUserId
};
