import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { Utils } from "../../../utills/helperMethods";
import { HTTPStatusCode, LoginTypes, OtpType, UserType, ValidateTypes } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import { config } from "../../../config";
import { StaticMessages } from "../../../utills/staticMessage";
import LoginSignupService from "./loginSignup.service";
import getErrorObject from "../../../utills/errorStructure";
import { jwtSign } from "../../../utills/token";
import { CustomError } from "../../../utills/types";
import userService from "../users/user.service";
import DistanceMatchService from "../distanceMatch/distanceMatch.service";
import moment from "moment";

const userLogin = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const eMail = req.body.email ? req.body.email.toLowerCase() : req.body.email;
        let jwtToken: any = await LoginSignupService.userLogin(req.body.userName, Utils.base64Decode(req.body.password), eMail, UserType.USER);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.LOGIN_SUCCESS,
            jwt_token: jwtToken
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.LOGIN_FAILED,
            status: Constants.ERROR
        });
    }
};

const createUser = async (req: Request, res: Response, next: NextFunction) => {
    try {
        //precheck
        const eMail = req.body.email ? req.body.email.toLowerCase() : req.body.email;
        let respSuccessMessage = "";
        req.body.loginType = req.body.signUpType;
        if (String(req.body.signUpType).toUpperCase() === LoginTypes.NORMAL) {
            if (req.body.password == "" || typeof req.body.password == 'undefined') {
                throw new CustomError(StaticMessages.PASSWORD_MISSING, HTTPStatusCode.BAD_REQUEST)
            }
            respSuccessMessage = StaticMessages.SIGNUP_RESET_OTP_SENT + ` ${Utils.maskEmail(eMail)} from ${config.dev.smtp.from_email}`
        } else if (String(req.body.signUpType).toUpperCase() === LoginTypes.GOOGLE) {
            if (req.body.googleId == "" || typeof req.body.googleId == 'undefined') {
                throw new CustomError(StaticMessages.getItemMissing('Google Id'), HTTPStatusCode.BAD_REQUEST)
            } else {
                req.body.password = req.body.googleId;
            }
            req.body.isVerified = 1;
            respSuccessMessage = StaticMessages.LOGIN_DETAIL_SENT + ` ${Utils.maskEmail(eMail)} from ${config.dev.smtp.from_email}`
        } else if (String(req.body.signUpType).toUpperCase() === LoginTypes.FACEBOOK) {
            if (req.body.facebookId == "" || typeof req.body.facebookId == 'undefined') {
                throw new CustomError(StaticMessages.getItemMissing('Facebook Id'), HTTPStatusCode.BAD_REQUEST)
            } else {
                req.body.password = req.body.facebookId;
            }
            req.body.isVerified = 1;
            respSuccessMessage = StaticMessages.LOGIN_DETAIL_SENT + ` ${Utils.maskEmail(eMail)} from ${config.dev.smtp.from_email}`
        } else if (String(req.body.signUpType).toUpperCase() === LoginTypes.APPLE) {
            if (req.body.appleId == "" || typeof req.body.appleId == 'undefined') {
                throw new CustomError(StaticMessages.getItemMissing('Google Id'), HTTPStatusCode.BAD_REQUEST)
            } else {
                req.body.password = req.body.appleId;
            }
            req.body.isVerified = 1;
            respSuccessMessage = StaticMessages.LOGIN_DETAIL_SENT + ` ${Utils.maskEmail(eMail)} from ${config.dev.smtp.from_email}`
        }
        if (req.body.userName) {
            let isUserNameFound = await LoginSignupService.checkUserByUserName(req.body.userName);
            if (isUserNameFound && isUserNameFound.isVerified == 1) {
                throw new CustomError(StaticMessages.USERNAME_UNIQUE, HTTPStatusCode.CONFLICT)
            }
        } else {
            req.body.userName = ""
        }
        if (req.body.countryCode && req.body.phone) {
            let isPhoneFound = await LoginSignupService.checkUserByPhone(req.body.countryCode, req.body.phone);
            if (isPhoneFound && isPhoneFound.isVerified == 1) {
                throw new CustomError(StaticMessages.PHONE_UNIQUE, HTTPStatusCode.CONFLICT)
            }
        }
        let isEmailFound = await LoginSignupService.checkUserByEmail(eMail);
        if (isEmailFound && isEmailFound.isVerified == 1 && String(req.body.signUpType).toUpperCase() === LoginTypes.NORMAL) {
            throw new CustomError(StaticMessages.EMAIL_UNIQUE, HTTPStatusCode.CONFLICT)
        } else if (isEmailFound && isEmailFound.isVerified == 0 && String(req.body.signUpType).toUpperCase() === LoginTypes.NORMAL) {
            userService.updatePassword({ oldPassword: isEmailFound.password, password: req.body.password }, isEmailFound.id)
            const jwtToken = jwtSign({ name: isEmailFound.userName, email: isEmailFound.email, id: isEmailFound.id, userType: isEmailFound.userType, issueTimeStamp: moment().unix() });
            // LoginSignupService.createOtpAndSendEmail({ email: isEmailFound.email });
            await LoginSignupService.logUserToken(isEmailFound.id, isEmailFound.userType, jwtToken);

            return res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                message: respSuccessMessage,
                jwt_token: jwtToken
            });
        } else {
            req.body.emailNotification = 1;
            let user = await LoginSignupService.createUser(req.body);
            await DistanceMatchService.createOrUpdateDistanceMatch({
                userId: user.id,
                distanceInKms: 0,
                isEnabled: 0
            });
            const jwtToken = jwtSign({ name: user.userName, email: user.email, id: user.id, userType: user.userType, issueTimeStamp: moment().unix() });
            await LoginSignupService.logUserToken(user.id, user.userType, jwtToken);
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                message: respSuccessMessage,
                jwt_token: jwtToken
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error)
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.SIGNUP_FAILED
        });
    }
};

const validateOTP = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const eMail = req.body.email ? req.body.email.toLowerCase() : req.body.email;
        await LoginSignupService.validateOTP(eMail, req.body.otp, OtpType.SIGNUP);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.OTP_VERIFIED
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: error.messages,
            status: Constants.ERROR
        });
    }
};

const resendOTP = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const eMail = req.body.email ? req.body.email.toLowerCase() : req.body.email;
        let updateResponse = await LoginSignupService.resendOTP(eMail);
        if (updateResponse) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.SIGNUP_RESET_OTP_SENT + ` ${Utils.maskEmail(eMail)} from ${config.dev.smtp.from_email}`,
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.OP_FAILED,
                error: {
                    message: StaticMessages.OP_FAILED,
                    statusCode: HTTPStatusCode.BAD_REQUEST
                },
                status: Constants.ERROR
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.OP_FAILED,
            status: Constants.ERROR
        });
    }
};

const validateEmailUsernamePhone = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userFound = false;
        const validateType = parseInt(req.body.validateType);
        switch (validateType) {
            case ValidateTypes.EMAIL:
                userFound = await userService.exists({
                    email: req.body.validateValue,
                    isVerified: 1
                });
                break;
            case ValidateTypes.USERNAME:
                userFound = await userService.exists({
                    userName: req.body.validateValue,
                    isVerified: 1
                });
                break;
            case ValidateTypes.PHONE:
                userFound = await userService.exists({
                    phone: req.body.validateValue,
                    isVerified: 1
                });
                break;
            default:
                throw new CustomError(StaticMessages.INVALID_VALIDATE_TYPE, HTTPStatusCode.BAD_REQUEST)
                break;
        }
        if (userFound) {
            switch (validateType) {
                case ValidateTypes.EMAIL:
                    throw new CustomError(StaticMessages.EMAIL_UNIQUE, HTTPStatusCode.BAD_REQUEST)
                    break;
                case ValidateTypes.USERNAME:
                    throw new CustomError(StaticMessages.USERNAME_UNIQUE, HTTPStatusCode.BAD_REQUEST)
                    break;
                case ValidateTypes.PHONE:
                    throw new CustomError(StaticMessages.PHONE_UNIQUE, HTTPStatusCode.BAD_REQUEST)
                    break;
                default:
                    throw new CustomError(StaticMessages.INVALID_VALIDATE_TYPE, HTTPStatusCode.BAD_REQUEST)
                    break;
            }
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getItemMessage('User not found with this validation'),
            status: Constants.SUCCESS
        });


    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.OP_FAILED,
            status: Constants.ERROR
        });
    }
};

const userLogOut = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];
        const user = await userService.getUserById(authUser.id, ['id', 'userType']);
        if (user) {
            await LoginSignupService.userLogout(user.id, user.userType, <string>req.header('Access-Token'));
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.LOGOUT_SUCCESS
            });
        } else {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }

    } catch (error) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.LOGOUT_FAILED,
            status: Constants.ERROR
        });
    }
};

export default {
    userLogin,
    createUser,
    validateOTP,
    resendOTP,
    validateEmailUsernamePhone,
    userLogOut
}
