import { NextFunction, Request, Response } from "express";
import Joi, { valid } from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";

const signUpValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("LoginSignUp Validator  -> SignUp Validator")
    try {
        const schema = Joi.object({
            signUpType: Joi.number().allow(1, 2, 3, 4).required().description('signUpType: 1-Normal, 2-Facebook, 3-Google, 4-Apple').error(new Error('signUpType is missing/invalid')),
            email: Joi.string().required().description('Email').error(new Error('Email is missing')),
            userName: Joi.string().required().description('userName').error(new Error('userName is missing')),
            countryCode: Joi.string().required().description('countryCode').error(new Error('countryCode is missing')),
            phone: Joi.string().required().description('phone').error(new Error('phone is missing')),
            password: Joi.string().description('Password').when('signUpType', {
                is: [1],
                then: Joi.string().required().description('customer email address').example('rinkesh@mobifyi.com'),
                otherwise: Joi.string().optional().allow('')
            }),
            termOfServicesConsent: Joi.number().allow(0, 1).required().description('termOfServicesConsent: 0-False, 1-True').error(new Error('termOfServicesConsent is missing/invalid')),
            facebookId: Joi.string().allow('').default('').description('facebook id').example('32446889756543'),
            appleId: Joi.string().allow('').default('').description('apple id').example('32446889756543'),
            googleId: Joi.string().allow('').default('').description('google id').example('2131321321311')
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const validateEmailUserNamePhoneValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("LoginSignUp Validator  -> Validate Email, Phone, Username")
    try {
        const schema = Joi.object({
            validateType: Joi.number().allow(1, 2, 3).required().description('validateType').error(new Error('validateType is missing')),
            validateValue: Joi.string().allow('').required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const loginValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("LoginSignUp Validator  -> Validate Email, password")
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            email: Joi.string().required().description('Email').error(new Error('Email is missing')),
            password: Joi.string().required().description('password').error(new Error('password is missing'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        };
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const validateRegistrationOTP = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            email: Joi.string().allow('').description('email').required(),
            otp: Joi.string().allow('').description('otp').required()
        })

        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const resendOTP = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            requestHeader: Joi.object().optional().description('requestHeader').error(new Error('Request Header is missing')),
            email: Joi.string().allow('').description('email').required()
        })

        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};
export default {
    signUpValidator,
    validateEmailUserNamePhoneValidator,
    loginValidator,
    validateRegistrationOTP,
    resendOTP
}