import { Request, Response } from "express";
import { HTTPStatusCode, PgResponseCases } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import PaymentGatewayLogs from "./paymentGateway.service";
import UserService from "../users/user.service";
import { createCustomerProfile, createCustomerPaymentProfile, getCustomerProfile, deleteCustomerPaymentProfile } from "../../commonServices/paymentGatewayService";
import { CustomError } from "../../../utills/types";
import paymentGatewayService from "./paymentGateway.service";

const addNewCard = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id, ["firstName", "lastName", "phone", "email", "pgProfileId"]);
        if (!userData) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
        const paymentLogsData: any = {
            userId: user.id
        };
        const pgServiceData = {
            firstName: userData.firstName,
            lastName: userData.lastName,
            phone: userData.phone,
            email: userData.email,
            cardNumber: req.body.cardNumber,
            cardCode: req.body.cvv,
            cardExpiryDate: req.body.cardExpiryDate,
            pgProfileId: userData.pgProfileId,
        }
        const isPgProfileExists = await PaymentGatewayLogs.getLogsByUserId(user.id);
        if (isPgProfileExists.length > 0) { //if exists then add payment profile only
            const profileData = await createCustomerPaymentProfile(pgServiceData);
            if (profileData && profileData['messages']['resultCode'] == PgResponseCases.SUCCESS) {
                paymentLogsData.pgProfileId = profileData['customerProfileId'];
                paymentLogsData.pgPaymentProfileId = profileData['customerPaymentProfileId'];
                await PaymentGatewayLogs.createLogs(paymentLogsData);
                await UserService.updateUser(user.id, { pgProfileId: profileData['customerProfileId'] });

            } else if (profileData && profileData['messages']['resultCode'] == PgResponseCases.ERROR && profileData['messages']['message'][0].text.includes('duplicate')) {
                throw new CustomError(StaticMessages.CARD_DUBPLICATE_ERROR, HTTPStatusCode.CONFLICT);
            } else if (profileData && profileData['messages']['resultCode'] == PgResponseCases.ERROR && profileData['messages']['message'][0].text.includes('(TESTMODE)')) {
                throw new CustomError(profileData['messages']['message'][0].text.replace('(TESTMODE) ', ''), HTTPStatusCode.BAD_REQUEST);
            } else {
                throw new CustomError(profileData['messages']['message'][0].text.replace('(TESTMODE) ', ''), HTTPStatusCode.BAD_REQUEST);
            }
        } else {  // else create customer profile along with payment details
            let profileData = await createCustomerProfile(pgServiceData);

            if (profileData && profileData['messages']['resultCode'] == PgResponseCases.SUCCESS) {
                paymentLogsData.pgProfileId = profileData['customerProfileId'];
                paymentLogsData.pgPaymentProfileId = profileData['customerPaymentProfileIdList'][0];

                await PaymentGatewayLogs.createLogs(paymentLogsData);
                await UserService.updateUser(user.id, { pgProfileId: profileData['customerProfileId'] });

            } else if (profileData && profileData['messages']['resultCode'] == PgResponseCases.ERROR && profileData['messages']['message'][0].text.includes('duplicate')) {
                throw new CustomError(StaticMessages.CARD_DUBPLICATE_ERROR, HTTPStatusCode.CONFLICT);
            } else if (profileData && profileData['messages']['resultCode'] == PgResponseCases.ERROR && profileData['messages']['message'][0].text.includes('(TESTMODE)')) {
                throw new CustomError(profileData['messages']['message'][0].text.replace('(TESTMODE) ', ''), HTTPStatusCode.BAD_REQUEST);
            } else {
                throw new CustomError(profileData['messages']['message'][0].text.replace('(TESTMODE) ', ''), HTTPStatusCode.BAD_REQUEST);
            }
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.CARD_CREATED,
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: StaticMessages.CARD_CREATE_ERROR
        });
    }
};

const getCards = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id, ["firstName", "lastName", "phone", "email", "pgProfileId"]);
        if (!userData) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }
        const storedCards = await PaymentGatewayLogs.getLogsByUserId(user.id);
        if (storedCards.length == 0) throw new CustomError(StaticMessages.CARDS_NOT_FOUND, HTTPStatusCode.NOTFOUND);
        const cards = [];
        const pgCardsData = await getCustomerProfile(userData.pgProfileId);

        if (pgCardsData && pgCardsData["messages"] && pgCardsData["messages"].resultCode == PgResponseCases.SUCCESS) {
            const paymentProfiles = pgCardsData['profile'].paymentProfiles;
            paymentProfiles.map((pgProfile) => {
                for (let i = 0; i < storedCards.length; i++) {
                    if (pgProfile.customerPaymentProfileId == storedCards[i].pgPaymentProfileId) {
                        cards.push({
                            pgPaymentProfileId: pgProfile.customerPaymentProfileId,
                            ...pgProfile.payment.creditCard
                        });
                    }
                }
            });

        } else {
            throw new CustomError(pgCardsData['messages'].message[0].text, HTTPStatusCode.BAD_REQUEST);
        }
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.CARD_CREATED,
            cards: cards
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const deleteCards = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id, ["id", "pgProfileId"]);
        if (!userData) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }

        const storedCards = await PaymentGatewayLogs.getLogsByUserIdProfileIdPaymentId(user.id, userData.pgProfileId, req.body.paymentProfileId);
        if (!storedCards) throw new CustomError(StaticMessages.CARDS_NOT_FOUND, HTTPStatusCode.NOTFOUND);

        const pgDeleteResp = await deleteCustomerPaymentProfile(storedCards.pgProfileId, storedCards.pgPaymentProfileId);
        if (pgDeleteResp && pgDeleteResp['messages']['resultCode'] == PgResponseCases.SUCCESS) {
            await paymentGatewayService.deletePaymentProfile(user.id, storedCards.pgProfileId, storedCards.pgPaymentProfileId);
        } else {
            throw new CustomError(StaticMessages.CARD_DELETE_ERROR, HTTPStatusCode.BAD_REQUEST);
        }
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.CARD_DELETED,
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: StaticMessages.CARD_DELETE_ERROR
        });
    }
};

export default {
    addNewCard,
    getCards,
    deleteCards
}