import express, { NextFunction, Request, Response } from 'express';
import { Constants } from '../../../utills/constants';
import { HTTPStatusCode } from '../../../utills/enums';
import getErrorObject from '../../../utills/errorStructure';
import { StaticMessages } from '../../../utills/staticMessage';
import { Logging } from '../../commonServices/logging.service';
import PostService from './posts.service';
import getErrorMessage from "../../../utills/errorStructure";

const addPost = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let createPostData = await PostService.createPost(user.id, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getCreatedMessage('Post'),
            post: createPostData,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.POST_CREATE_FAILED,
            status: Constants.ERROR
        });
    }
}

const getFriendsPosts = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let createPostData = await PostService.getFriendsPosts(user.id, String(req.query.userId), parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
        res.status(HTTPStatusCode.SUCCESS).json({
            post: createPostData,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Post'),
            status: Constants.ERROR
        });
    }
};

const getPostById = async (req: Request, res: Response) => {
    try {
        let createPostData = await PostService.getPostById(req.params.id);
        res.status(HTTPStatusCode.SUCCESS).json({
            post: createPostData,
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getNotExistMessage('Post'),
            status: Constants.ERROR
        });
    }
};

const deletePost = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let deletePostResult = await PostService.deletePost(req.params.id, user.id);
        if (deletePostResult > 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getDeletedMessage('Post'),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.NO_RECORD_FOUND,
                status: Constants.SUCCESS
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getDeletedFailedMessage('Post'),
            status: Constants.ERROR
        });
    }
};

const updatePost = async (req: Request, res: Response) => {
    try {
        // have to remove request in JOI Validation Middleware
        let authUser = req['user'];
        let updatePostResult = await PostService.updatePost(req.params.id, authUser.id, req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getUpdatedMessage('Post'),
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getUpdateFailedMessage('Post'),
            status: Constants.ERROR
        });
    }
};

const updateComment = async (req: Request, res: Response) => {
    try {
        let user = req["user"]
        let updateCommentResult = await PostService.updateComment(req.params.id, user.id, req.body);
        if (updateCommentResult > 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getUpdatedMessage('Comment'),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getUpdateFailedMessage('Comment'),
                status: Constants.SUCCESS
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getDeletedFailedMessage('Comment'),
            status: Constants.ERROR
        });
    }
};

const addComments = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        if (req.params.commentId) {
            req.body['parentId'] = req.params.commentId;
        }
        req.body['PostId'] = req.params.id;
        req.body['UserId'] = user.id;
        req.body['postUserId'] = req.body.postUserId;
        let updatePostResult = await PostService.addComments(req.body);
        res.status(HTTPStatusCode.SUCCESS).json({
            message: StaticMessages.getCreatedMessage('Comment'),
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.COMMENT_NOT_ADDED,
            status: Constants.ERROR
        });
    }
};

const deleteComment = async (req: Request, res: Response) => {
    try {
        let user = req['user'];
        let deletePostResult = await PostService.deleteComment(req.params.commentId, user.id);
        if (deletePostResult > 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getDeletedMessage('Comment'),
                status: Constants.SUCCESS
            });
        } else {
            res.status(HTTPStatusCode.SUCCESS).json({
                message: StaticMessages.getDeletedFailedMessage('Comment'),
                status: Constants.SUCCESS
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error);
        error = getErrorObject(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.getDeletedFailedMessage('Comment'),
            status: Constants.ERROR
        });
    }
};

const getComments = async (req: Request, res: Response) => {
    try {
        let comments = []
        if (req.query.limit && req.query.offset) {
            comments = await PostService.getComments(req.params.id, parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
        } else {
            comments = await PostService.getComments(req.params.id);
        }
        res.status(HTTPStatusCode.SUCCESS).json({
            comments: comments,
            message: StaticMessages.getItemMessage("Comments List."),
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({ message: error });
    }
};

const addReactionOnComment = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        data['CommentUserId'] = req.body.userId;
        data['postId'] = req.body.postId;
        data['CommentId'] = req.body.commentId;
        data['reactionType'] = req.body.reactionType ? req.body.reactionType : 'like';
        await PostService.addCommentReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_FAIL,
            status: Constants.ERROR
        });
    }
};

const addReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        data['PostUserId'] = req.body.userId;
        data['PostId'] = req.body.postId;
        data['CommentId'] = req.body.commentId;
        data['reactionType'] = req.body.reactionType ? req.body.reactionType : 'like';
        await PostService.addReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_FAIL,
            status: Constants.ERROR
        });
    }
};

const removeReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        if (req.body.postId) {
            data['PostId'] = req.body.postId;
        }
        await PostService.removeReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_REMOVE_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_REMOVE_FAIL,
            status: Constants.ERROR
        });
    }
};

const removeReactionOnComment = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        if (req.body.postId) {
            data['CommentId'] = req.body.commentId;
        }
        await PostService.removeReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_REMOVE_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_REMOVE_FAIL,
            status: Constants.ERROR
        });
    }
};

export default {
    addPost,
    deletePost,
    updatePost,
    getFriendsPosts,
    getPostById,
    addComments,
    deleteComment,
    getComments,
    updateComment,
    addReaction,
    addReactionOnComment,
    removeReaction,
    removeReactionOnComment
}