import { NextFunction, Request, Response } from "express";
import Joi from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
//posts
const addPost = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.string().required().error(new Error('Request Header is missing')),
            media: Joi.binary(),
            location: Joi.string(),
            files: Joi.array(),
            mediaType: Joi.number().required(),
            postText: Joi.string(),
            tags: Joi.array()
        });
        if (req['files']) {
            req.body['files'] = req['files'];
        }
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const updatePost = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            location: Joi.string(),
            files: Joi.array(),
            mediaType: Joi.number(),
            postText: Joi.string(),
            tags: Joi.array(),
            requestHeader: Joi.string().required().error(new Error('Request Header is missing'))
        });
     
        if (req['files']) {
            req.body['files'] = req['files'];
        }
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

//comments
const addComments = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.object().required().error(new Error('Request Header is missing')),
            text: Joi.string().required(),
            postUserId: Joi.string().required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const updateComment = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.object().required().error(new Error('Request Header is missing')),
            text: Joi.string().required(),
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

//reactions
const addReaction = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.object().required().error(new Error('Request Header is missing')),
            postId: Joi.string(),
            storyId: Joi.string(),
            commentId: Joi.string(),
            userId: Joi.string().required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const addReactionOnComment = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.object().required().error(new Error('Request Header is missing')),
            commentId: Joi.string().required(),
            userId: Joi.string().required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const removeReactionFromPost = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.object().required().error(new Error('Request Header is missing')),
            postId: Joi.string().required()
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const removeReactionFromComment = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            requestHeader: Joi.object().required().error(new Error('Request Header is missing')),
            files: Joi.array(),
            commentId: Joi.string().required()
        });
     
        if (req['files']) {
            req.body['files'] = req['files'];
        }
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    addPost,
    updatePost,
    addComments,
    updateComment,
    addReaction,
    addReactionOnComment,
    removeReactionFromPost,
    removeReactionFromComment
}   
