import { Request, Response, NextFunction } from "express";
import ReportedUsersService from "./reportedUsers.service";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import { FolderNames, HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import moment from "moment";
import UserService from "../users/user.service";

const addReport = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let user = req['user'];
        let date = moment().format('MM DD YYYY, HH:mm:ss');
        const userData = {
            userId: user.id,
            comment: req.body.comment,
            lastReportedTime: date,
            action: "false",
            reportReason: req.body.reportReason,
            toUserId: req.body.toUserId
        };
        let report = await ReportedUsersService.addReport(userData);
        if (report) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                report: report,
                message: StaticMessages.getCreatedMessage('Report')
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const getReportList = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let report = await ReportedUsersService.getReportList( parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
        if (report) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                report: report,
                message: StaticMessages.getListReturnedMessage('Report')
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const getUserReportDetail = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let reponse = {};
        const userDetails = await UserService.getUserById(req.params.id);
        userDetails.profilePicture = userDetails.profilePicture ? '/static/users/' + req.params.id + '/' + FolderNames.PROFILE_PICTURE + '/' + userDetails.profilePicture : "";
        userDetails.profileThumbnail = userDetails.profileThumbnail ? '/static/users/' + req.params.id + '/' + FolderNames.PROFILE_PICTURE + '/' + userDetails.profileThumbnail : "";
        let reportedByMe = await ReportedUsersService.getReportedUserByMe(req.params.id);
        let reported = await ReportedUsersService.getReportedUser(req.params.id);
        reponse['userDetails'] = userDetails;
        reponse['reportedByMe'] = reportedByMe.length > 0 ? reportedByMe : [];
        reponse['reported'] = reported.length > 0 ? reported : [];
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getListReturnedMessage('Report Details'),
            reportList: reponse
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const getReportListByUsersId = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let report = await ReportedUsersService.getReportListByUsersId(req.params.id);
        if (report) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                reports: report,
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const deleteReportByUsersId = async (req: Request, res: Response, next: NextFunction) => {
    try {
        const user = req['user'];
        let report = await ReportedUsersService.deleteReportByUsersId(user.id, req.params.id);
        if (report) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                report: report,
                message: StaticMessages.getDeletedMessage('Report')
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const takeAction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let report = await ReportedUsersService.takeAction(req.params.id, req.body.action);
        if (report) {
            res.status(HTTPStatusCode.CREATED).json({
                status: Constants.SUCCESS,
                report: report,
                message: StaticMessages.ACTION_TAKEN
            });
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

const getSearchedReport = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let userName = req.query.userName || "";
        let action = req.query.action || "";
        let fromDate = req.query.fromDate || "";
        let toDate = req.query.toDate || "";
        let users = await ReportedUsersService.getSearchedReport(userName, action, String(fromDate), String(toDate));
        if (users.length == 0) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage("Users"),
                status: Constants.ERROR,
                error: {
                    statusCode: HTTPStatusCode.NOTFOUND,
                    message: StaticMessages.getNotExistMessage("Users"),
                }
            });
        }

        return res.status(HTTPStatusCode.SUCCESS).json({
            users: users,
            message: StaticMessages.getListReturnedMessage("User"),
            status: Constants.SUCCESS
        });
    } catch (error: any) {
        Logging.logRequestError(req, error);
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({ message: error });
    }
};

export default {
    addReport,
    getReportList,
    getReportListByUsersId,
    deleteReportByUsersId,
    takeAction,
    getUserReportDetail,
    getSearchedReport
}