import db from "../../../models";
import sequelize, { Op, Sequelize } from "sequelize";
import moment from "moment";
import { Logging } from "../../commonServices/logging.service";
const ReportedUser = db.ReportedUser;
const SubscribedUser = db.SubscribedUser;
const User = db.User;
const DatePageAction = db.DatePageAction;

const addReport = async (data) => {
    try {
        await DatePageAction.destroy({
            where: {
                [Op.or] : [{
                    userId: data.userId,
                    actionUserId: data.toUserId
                }, {
                    userId: data.toUserId,
                    actionUserId: data.userId
                }]
            }
        });
        return await ReportedUser.create(data);
    } catch (error: any) {
        return error;
    }
};

const getReportList = async (limit = 20, offset = 0) => {
    try {
        let reportList = await ReportedUser.findAll({
            attributes: [
                [Sequelize.fn('COUNT', Sequelize.col('toUserId')), 'totalReports'],
                ["toUserId", "reportedUserId"]
            ],
            group: ["reportedUserId"],
            offset: offset,
            limit: limit,
            raw: true
        });
        reportList = await Promise.all(await reportList.map(async (rpList) => {
            try {
                let subReport = await ReportedUser.findAll({
                    where: {
                        toUserId: rpList.reportedUserId
                    },
                    attributes: [
                        "comment",
                        "lastReportedTime",
                        "action"
                    ],
                    order: [['lastReportedTime', 'DESC']],
                    raw: true
                });
                rpList.lastComment = subReport[0].comment;
                rpList.lastReportedTime = subReport[0].lastReportedTime;
                rpList.lastAction = subReport[0].action == 1;

                let subscriptionDetail = await SubscribedUser.findAll({
                    where: {
                        expiresOn: {
                            [Op.gt]: moment().unix()
                        },
                        userId: rpList.reportedUserId
                    },
                    include: [{
                        model: db.SubscriptionPlan,
                        attributes: [
                            "planName"
                        ]
                    }],
                    order: [['id', 'DESC']],
                    raw: true
                });

                if (subscriptionDetail.length > 0) {
                    rpList.planName = subscriptionDetail[0]['SubscriptionPlan.planName'];
                } else {
                    rpList.planName = "FREE USER";
                }

                let userDetail = await User.findOne({
                    where: {
                        id: rpList.reportedUserId
                    },
                    attributes: [
                        "userName",
                        "firstName",
                        "lastName",
                    ]
                });

                rpList.userName = userDetail.userName;
                rpList.firstName = userDetail.firstName || "";
                rpList.lastName = userDetail.lastName || "";



            } catch (err) {
                Logging.logMessage(`REPORTLIST PROMISE ERR...${err}`)
            }

            return rpList;
        }))
        let count = await ReportedUser.count();
        return { reportList, count };
    } catch (error) {
        return error;
    }
};


const getReportedUserByMe = async (userId: string) => {
    try {
        let reports = await ReportedUser.findAll({
            where: {
                userId: userId
            },
            include: [
                {
                    model: db.User,
                    attributes: [
                        "userName"
                    ],
                    on: Sequelize.literal('User.id = ReportedUser.toUserId')
                }
            ],
            attributes: [
                [sequelize.col('User.userName'), 'userName'],
                "comment",
                "lastReportedTime",
                "reportReason",
                "toUserId",
                "userId"
            ],
            order: [["lastReportedTime", "DESC"]],
            raw: true
        });
        reports.map((rp) => delete rp['User.userName']);
        return reports;
    } catch (error) {
        return error;
    }
};

const getReportedUser = async (userId: string) => {
    try {
        let reports = await ReportedUser.findAll({
            where: {
                toUserId: userId
            },
            include: [
                {
                    model: db.User,
                    attributes: [
                        "userName"
                    ],
                    on: Sequelize.literal('User.id = ReportedUser.userId')
                }
            ],
            attributes: [
                [sequelize.col('User.userName'), 'userName'],
                "comment",
                "lastReportedTime",
                "reportReason",
                "toUserId",
                "userId"
            ],
            order: [["lastReportedTime", "DESC"]],
            raw: true
        });
        reports.map((rp) => delete rp['User.userName']);
        return reports;
    } catch (error) {
        return error;
    }
};

const getReportListByUsersId = async (id) => {
    try {
        let reportList = await ReportedUser.findAll({
            where: {
                toUserId: id
            },
            include: [{
                model: db.User,
                as: 'User',
                attributes:[],
                on: Sequelize.literal('User.id = ReportedUser.toUserId')
            },{
                model: db.User,
                as: 'User',
                attributes:[],
                on: Sequelize.literal('User.id = ReportedUser.userId')
            }],
            attributes: [
                'id',
                'comment',
                'reportReason',
                'action',
                'userId',
                'toUserId',
                [Sequelize.col('User.userName'), 'reportedUserName'],
                [Sequelize.col('User.firstName'), 'reportedUserFirstName'],
                [Sequelize.col('User.lastName'), 'reportedUserLastName'],
                [Sequelize.col('User.countryCode'), 'reportedUserCountryCode'],
                [Sequelize.col('User.phone'), 'reportedUserPhone'],
                [Sequelize.col('User.gender'), 'reportedUserGender'],
                [Sequelize.col('User.DOB'), 'reportedUserDOB'],
                [Sequelize.col('User.userType'), 'reportedUserType'],
                [Sequelize.col('User.profilePicture'), 'reportedUserProfilePicture'],

                [Sequelize.col('User.userName'), 'reportedByUserName'],
                [Sequelize.col('User.firstName'), 'reportedByUserFirstName'],
                [Sequelize.col('User.lastName'), 'reportedByUserLastName'],
                [Sequelize.col('User.countryCode'), 'reportedByUserCountryCode'],
                [Sequelize.col('User.phone'), 'reportedByUserPhone'],
                [Sequelize.col('User.gender'), 'reportedByUserGender'],
                [Sequelize.col('User.DOB'), 'reportedByUserDOB'],
                [Sequelize.col('User.userType'), 'reportedByUserType'],
                [Sequelize.col('User.profilePicture'), 'reportedByUserProfilePicture'],
            ],
            raw: true
        });
        return reportList;
    } catch (error) {
        return error;
    }
};

const deleteReportByUsersId = async (id, reportedUserId) => {
    try {
        let reportList = await ReportedUser.destroy({
            where: {
                userId: id,
                toUserId: reportedUserId
            }
        });
        return reportList;
    } catch (error) {
        return error;
    }
};

const takeAction = async (id, action) => {
    try {
        let actionTaken = await ReportedUser.update({ action }, {
            where: {
                toUserId: id
            }
        })
        return actionTaken;
    } catch (error) {
        return error;
    }
};

const getSearchedReport = async (userName, action, fromDate = "", toDate = "") => {
    try {
        const filteredUsers = await User.findAll({
            where: {
                [Op.or]: [
                    { userName: { [Op.regexp]: `.*${userName}.*` } },
                ]
            },
            attributes: ["id"],
            raw: true
        });
        const userIds = [];
        filteredUsers.map((user) => userIds.push(user.id));
        const reportListCond = {
            where: {
                toUserId: {
                    [Op.in]: userIds
                }
            },
            attributes: [
                [Sequelize.fn('COUNT', Sequelize.col('toUserId')), 'totalReports'],
                ["toUserId", "reportedUserId"]
            ],
            group: ["reportedUserId"],
            raw: true
        }
        let lastReportedTime = {}
        if (fromDate !== "") {
            lastReportedTime = {
                [Op.gte]: moment(fromDate, "MM DD YYYY")
            }
        };
        if (toDate !== "") {
            lastReportedTime = {
                [Op.lte]: moment(toDate, "MM DD YYYY")
            }
        };
        if (fromDate !== "" && toDate !== "") {
            lastReportedTime = {
                [Op.gte]: moment(fromDate, "MM DD YYYY"),
                [Op.lte]: moment(toDate, "MM DD YYYY")
            }
        }
        if(fromDate !== "" || toDate !== "") reportListCond.where['lastReportedTime'] = lastReportedTime;

        if (action !== '') reportListCond.where['action'] = parseInt(action);
        let reportList = await ReportedUser.findAll(reportListCond);
        reportList = Promise.all(await reportList.map(async (rpList) => {
            try {
                const subReportCond = {
                    where: {
                        toUserId: rpList.reportedUserId
                    },
                    include: [
                        {
                            model: db.User,
                            attributes: [
                                "userName",
                                "firstName",
                                "lastName"
                            ],
                            on: Sequelize.literal('User.id = ReportedUser.toUserId')
                        }
                    ],
                    attributes: [
                        "comment",
                        "lastReportedTime",
                        "action"
                    ],
                    order: [['lastReportedTime', 'DESC']],
                    raw: true
                }
                if(fromDate !== "" || toDate !== "") subReportCond.where['lastReportedTime'] = lastReportedTime;
                if (action !== '') subReportCond.where['action'] = parseInt(action);
                let subReport = await ReportedUser.findAll(subReportCond);
                rpList.lastComment = subReport[0].comment;
                rpList.lastReportedTime = subReport[0].lastReportedTime;
                rpList.lastAction = subReport[0].action;
                rpList.userName = subReport[0]["User.userName"];
                rpList.firstName = subReport[0]["User.firstName"] || "";
                rpList.lastName = subReport[0]["User.lastName"] || "";
                let subscriptionDetail = await SubscribedUser.findAll({
                    where: {
                        expiresOn: {
                            [Op.gt]: moment().unix()
                        },
                        userId: rpList.reportedUserId
                    },
                    include: [{
                        model: db.SubscriptionPlan,
                        attributes: [
                            "planName"
                        ]
                    }],
                    order: [['id', 'DESC']],
                    raw: true
                });
                if (subscriptionDetail.length > 0) {
                    rpList.planName = subscriptionDetail[0]['SubscriptionPlan.planName'];
                } else {
                    rpList.planName = "FREE USER";
                }
            } catch (err) {
                Logging.logMessage(`REPORTLIST PROMISE ERR...${err}`)
            }
            return rpList;
        }));
        return reportList;
    } catch (error: any) {
        throw error;
    }
};

export default {
    addReport,
    getReportList,
    getReportListByUsersId,
    deleteReportByUsersId,
    takeAction,
    getReportedUserByMe,
    getReportedUser,
    getSearchedReport
}
