import { Request, Response, NextFunction } from "express";
import { StaticMessages } from "../../../utills/staticMessage";
import { Logging } from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import { HTTPStatusCode, Questions } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import UserService from "../users/user.service";
import SignUpCheckListService from "./signUpCheckList.service";
import LocationService from "../locations/location.service";
import { CustomError } from "../../../utills/types";
import datingPageMediaService from "../datingPageMedia/datingPageMedia.service";
import userCharacteristicService from "../userCharacteristic/userCharacteristic.service";

/**
 *  mobileVerified: 409;
    documentVerified: 410;
    sexOffenderVerified: 411;
    locationAdded: 412;
    questionFilled: 413;
    datingPageMediaUploaded: 414; 
 */

const getStatus = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        const userData = await UserService.getUserById(authUser.id);

        let documentsUploaded = true;
        let genderFound = false;
        let userIdentityFound = false;
        let userDatingInterestFound = false;
        if (userData.isVerified) {
            await SignUpCheckListService.updateCheckListByUserId(authUser.id, { mobileVerified: true });
        } else {
            throw new CustomError(StaticMessages.getItemMessage("Mobile not verified yet"), HTTPStatusCode.CONFLICT);
        }

        if (userData.isDocumentVerified === 3 || userData.isDocumentVerified === '3' ) {
            await SignUpCheckListService.updateCheckListByUserId(authUser.id, { documentVerified: true });
        } else if (userData.isDocumentVerified === 0 || userData.isDocumentVerified === '0') {
            documentsUploaded = false;
            throw new CustomError(StaticMessages.getItemMessage("Documents not uploaded yet"), HTTPStatusCode.ERROR_410);
        }
        
        const locationData = await LocationService.getLocationByUserId(authUser.id);
        if (!locationData) {
            throw new CustomError(StaticMessages.getNotExistMessage("Location"), HTTPStatusCode.ERROR_412);
        } else {
            await SignUpCheckListService.updateCheckListByUserId(authUser.id, { locationAdded: true });
        }

        const userDatingInterestQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(authUser.id, Questions.I_AM_INTRESTED_IN_DATING);
        if (userDatingInterestQuestion.length > 0) {
            userDatingInterestFound = true;
        } else {
            throw new CustomError(StaticMessages.getItemMessage("Some Questions not Filled up yet"), HTTPStatusCode.ERROR_413);
        }

        const userIdentityQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(authUser.id, Questions.I_IDENTIFY_AS);
        if (userIdentityQuestion.length > 0) {
            userIdentityFound = true;
        } else {
            throw new CustomError(StaticMessages.getItemMessage("Some Questions not Filled up yet"), HTTPStatusCode.ERROR_413);
        }

        const datingPageMedia = await datingPageMediaService.getMediaByUserId(authUser.id);
        if (datingPageMedia.length == 0) {
            throw new CustomError(StaticMessages.getNotExistMessage("Dating Page Media"), HTTPStatusCode.ERROR_414);
        } else {
            await SignUpCheckListService.updateCheckListByUserId(authUser.id, { datingPageMediaUploaded: true });
        }

        if (userIdentityFound && userDatingInterestFound) {
            await SignUpCheckListService.updateCheckListByUserId(authUser.id, { questionFilled: true });
        }

        const signUpCheckListData = await SignUpCheckListService.getCheckListByUserId(authUser.id);

        if (!signUpCheckListData.mobileVerified) throw new CustomError(StaticMessages.getItemMessage("Mobile Not Verified yet"), HTTPStatusCode.CONFLICT);
        if (!documentsUploaded) throw new CustomError(StaticMessages.getItemMessage("Documents Not uploaded yet"), HTTPStatusCode.ERROR_410);
        // if (!signUpCheckListData.sexOffenderVerified) throw new CustomError(StaticMessages.getItemMessage("Sex Offender Check is Not Verified yet"), HTTPStatusCode.ERROR_411);
        if (!signUpCheckListData.locationAdded) throw new CustomError(StaticMessages.getItemMessage("Location not added yet"), HTTPStatusCode.ERROR_412);
        if (!signUpCheckListData.questionFilled) throw new CustomError(StaticMessages.getItemMessage("Some Questions not Filled up yet"), HTTPStatusCode.ERROR_413);
        if (!signUpCheckListData.datingPageMediaUploaded) throw new CustomError(StaticMessages.getItemMessage("Dating Page Media not uploaded yet"), HTTPStatusCode.ERROR_414);

        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.getItemMessage('Check List Done')
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: error.message
        });
    }
};

export default {
    getStatus,
}
