import {NextFunction, Request, Response} from "express";
import {
    CommonAnswers,
    FolderNames,
    HTTPStatusCode,
    ImageExentions,
    MediaSources,
    MediaTypes,
    Questions,
    VideoExtensions
} from "../../../utills/enums";
import {Constants} from "../../../utills/constants";
import {StaticMessages} from "../../../utills/staticMessage";
import path from 'path';
import {Logging} from "../../commonServices/logging.service";
import getErrorMessage from "../../../utills/errorStructure";
import StoryService from "./story.service";
import UserService from "../users/user.service";
import moment from "moment";
import {handleFileUpload, handleImageVideoThumb} from "../../commonServices/fsFileUpload";
import {CustomError} from "../../../utills/types";
import {v4 as uuidv4} from 'uuid';
import userCharacteristicService from "../userCharacteristic/userCharacteristic.service";

const addNewStory = async (req: Request, res: Response) => {
    try {
        const user = req['user'];
        let userData = await UserService.getUserById(user.id);
        if (!userData) {
            return res.status(HTTPStatusCode.NOTFOUND).json({
                message: StaticMessages.getNotExistMessage('User'),
                status: Constants.ERROR
            });
        }

        let uploadedFilePaths: any = []
        let thumbnailsPath: any = []
        let pathExt = null;
        if (req.body.mediaPath.length) {
            uploadedFilePaths = await handleFileUpload({
                type: MediaSources.STORY,
                userId: user.id,
                itemId: String(uuidv4()),
                files: req.body.mediaPath
            });

            thumbnailsPath = await handleImageVideoThumb({
                type: MediaSources.STORY,
                userId: user.id,
                itemId: null,
                files: uploadedFilePaths
            });

            pathExt = path.extname(uploadedFilePaths[0]);
        }

        let type = 3;
        let typeText = MediaTypes.TEXT;
        if (req.body.mediaPath.length && Object.values(ImageExentions).findIndex((imgEx) => imgEx == pathExt) > -1) {
            type = 1;
            typeText = MediaTypes.IMAGE;
        } else if (req.body.mediaPath.length && Object.values(VideoExtensions).findIndex((vidEx) => vidEx == pathExt) > -1) {
            type = 2;
            typeText = MediaTypes.VIDEO;
        };

        const storyInsertData = {
            UserId: parseInt(user.id),
            createdAt: new Date(),
            type: type,
            typeText: typeText,
            thumbnail: thumbnailsPath[0] || "",
            mediaPath: uploadedFilePaths[0] || "",
            timestamp: moment().unix(),
            expire: moment().add(1, 'days').unix(),
            totalViewsCount: 0,
            caption: req.body.caption || "",
        };
        let storyData = await StoryService.createStory(storyInsertData);
        storyData.dataValues.userId = storyData.dataValues.UserId;
        delete storyData.dataValues.UserId;
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.STORY_CREATED,
            story: storyData
        });
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            error: getErrorMessage(error),
            status: Constants.ERROR,
            message: StaticMessages.STORY_CREATE_FAILED
        });
    }
};

const getFollowingStories = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];
        let stories = await StoryService.getStories(authUser.id, req.query.limit, req.query.offset);
        if (stories.length) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.STORY_FOUND,
                stories: stories
            });
        } else {
            throw new CustomError(StaticMessages.NO_STORIES_FOUND, HTTPStatusCode.NOTFOUND);
        }
    } catch (error: any) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        return res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_STORIES_FOUND
        });
    }
};

const getStoriesByUserId = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise((resolve, reject) => {
        return UserService.getUserById(String(req.query.userId))
            .then(async (data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') })
                }

                const userBasicInfo = {
                    userName: data.userName,
                    firstName: data.firstName,
                    lastName: data.lastName,
                    profilePicture: data['profilePicture'] ? '/static/users/' + req.query.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + data['profilePicture'] : "",
                    profileThumbnail: data['profileThumbnail'] ? '/static/users/' + req.query.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + data['profileThumbnail'] : "",
                    age: data['DOB'] == null ? Constants.DEFAULT_AGE : moment().diff(data['DOB'], 'years'),
                    works: null,
                    educationLevel: null,
                    university: null,
                    hometown: null
                }

                const workQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(req.query.userId, Questions.WORKS);
                userBasicInfo.works = workQuestion[0] ? workQuestion[0].customAns : null;

                const eductaionLevelQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(req.query.userId, Questions.EDUCATION_LEVEL);
                const universityQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(req.query.userId, Questions.COLLEGE_UNIVERSITY);
                if (eductaionLevelQuestion.length > 0 && universityQuestion.length > 0) {
                    if (eductaionLevelQuestion[0].answers[0].answerText != CommonAnswers.PREFER_NOT_TO_SAY) {
                        userBasicInfo.educationLevel = eductaionLevelQuestion[0].answers[0].answerText;
                        userBasicInfo.university = universityQuestion[0].customAns;
                    }
                }

                const hometownQuestion = await userCharacteristicService.getCharacteristicsByUserIdQuestionId(req.query.userId, Questions.HOMETOWN);
                userBasicInfo.hometown = hometownQuestion[0] ? hometownQuestion[0].customAns : null;

                resolve(userBasicInfo)
            }).catch(err => reject(err))
    });

    const getUserStories = (userBasicInfo) => new Promise((resolve, reject) => {
        return StoryService.getStoriesByUserId(String(req.query.userId))
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt) => {
                        if (req.query.userId !== authUser.id) {
                            delete dt.totalViewsCount;
                        }

                        dt.url = '/static/users/' + dt.userId + '/' + FolderNames.STORIES + '/' + dt.url;
                        dt.thumbnail = '/static/users/' + dt.userId + '/' + FolderNames.STORIES + '/' + dt.thumbnail;
                        return dt;
                    })

                    resolve({ userBasicInfo: userBasicInfo, storyMedia: data });
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Stories') })
            }).catch((err) => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(getUserStories)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.STORY_FOUND,
                stories: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                err = getErrorMessage(err);
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const deleteStories = async (req: Request, res: Response) => {
    const authUser = req['user'];
    const getStoryData = () => new Promise<void>((resolve, reject) => {
        return StoryService.getStoriesByStoryIdUserId(req.body.storyId, authUser.id)
            .then((data) => {
                if (data.length > 0) resolve()
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Stories') })
            }).catch((err) => reject(err))
    });

    const deleteStory = () => new Promise<void>((resolve, reject) => {
        return StoryService.deleteStoriesByStoryIdUserId(req.body.storyId, authUser.id)
            .then(() => resolve())
            .catch((err) => reject(err))
    });

    return getStoryData()
        .then(deleteStory)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.STORY_DELETED
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};
const addStoryReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        data['StoryUserId'] = req.body.userId;
        data['StoryId'] = req.body.storyId;
        data['reactionType'] = req.body.reactionType ? req.body.reactionType : 'like';
        await StoryService.addStoryReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_FAIL,
            status: Constants.ERROR
        });
    }
};

const removeStoryReaction = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let authUser = req['user'];
        let data = {};
        data['UserId'] = authUser.id;
        if (req.body.storyId) {
            data['StoryId'] = req.body.storyId;
        }
        await StoryService.removeReaction(data);
        res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.REACTION_REMOVE_SUCCESS,
        });
    } catch (error: any) {
        Logging.logRequestError(req, "error.name" + error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            message: StaticMessages.REACTION_REMOVE_FAIL,
            status: Constants.ERROR
        });
    }
};

export default {
    addNewStory,
    getFollowingStories,
    getStoriesByUserId,
    deleteStories,
    addStoryReaction,
    removeStoryReaction
}
