import { Request, Response, NextFunction } from "express";
import { Logging } from "../../commonServices/logging.service";
import { HTTPStatusCode, PgResponseCases } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import getErrorMessage from "../../../utills/errorStructure";
import subscriptionPlanService from "./subscriptionPlan.service";
import { StaticMessages } from "../../../utills/staticMessage";
import moment from "moment";
import { chargeCustomerProfile } from "../../commonServices/paymentGatewayService";
import subscribedUserService from "../subscribedUser/subscribedUser.service";
import userService from "../users/user.service";
import paymentGatewayService from "../paymentGateway/paymentGateway.service";
import { CustomError } from "../../../utills/types";
import userPurchasesService from "../userPurchases/userPurchases.service";

const createSubscriptionPlan = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscriptionPlanService.createSubscriptionPlan(req.body);
        if (data) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                subscriptionPlan: data,
                message: StaticMessages.getCreatedMessage('Subscription Plan')
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

const getSubscriptionPlan = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscriptionPlanService.getSubscriptionPlan(parseInt(String(req.query.limit)), parseInt(String(req.query.offset)));
        if (data) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                subscriptionPlanList: data,
                message: StaticMessages.getListReturnedMessage('Subscription Plan')
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

const getSubscriptionPlanById = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscriptionPlanService.getSubscriptionPlanById(req.params.id);
        if (data) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                subscriptionPlan: data,
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

const updateSubscriptionPlan = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscriptionPlanService.updateSubscriptionPlanById(req.params.id, req.body);
        if (data.length > 0 && data[0] != 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                subscriptionPlan: data,
                message: StaticMessages.getUpdatedMessage('Subscription Plan')
            });
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

const deleteSubscriptionPlan = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let data = await subscriptionPlanService.deleteSubscriptionPlanById(req.params.id);
        if (data > 0) {
            res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                subscriptionPlan: data,
                message: StaticMessages.getDeletedMessage('Subscription Plan')
            });
        } else {
            res.status(HTTPStatusCode.NOTFOUND).json({
                status: Constants.ERROR
            })
        }
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(error.statusCode).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.NO_RECORD_FOUND
        });
    }
};

//User buys subscription plan
const buySubscriptionPlan = async (req: Request, res: Response, next: NextFunction) => {
    try {
        let user = req['user'];
        const userData = await userService.getUserById(user.id, ['id', 'pgProfileId']);
        const paymentProfileInfo = await paymentGatewayService.getLogsByUserIdProfileIdPaymentId(userData.id, userData.pgProfileId, req.body.paymentProfileId)

        if (!paymentProfileInfo) throw new CustomError(StaticMessages.CARDS_NOT_FOUND, HTTPStatusCode.NOTFOUND);

        let getplan = await subscriptionPlanService.getSubscriptionPlanById(req.body.subscriptionPlanId);
        if (!getplan) throw new CustomError(StaticMessages.getNotExistMessage('Subscription Plan'), HTTPStatusCode.NOTFOUND);
        const subscribedUserData = {
            planValidityInDays: getplan.planValidityInDays,
            activatedOn: moment().unix(),
            expiresOn: moment().add(parseInt(getplan.planValidityInDays), 'days').unix(),
            SubscriptionPlanId: getplan.id,
            UserId: userData.id
        };

        const chargeInfo = await chargeCustomerProfile(paymentProfileInfo.pgProfileId, paymentProfileInfo.pgPaymentProfileId, getplan.price, req.body.cvv);
        if (chargeInfo && chargeInfo['messages']['resultCode'] == PgResponseCases.SUCCESS) {
            const purchaseData = {
                txnDate: moment().unix(),
                txnId: chargeInfo['transactionResponse']['transId'],
                txnRefId: chargeInfo['refId'],
                paymentProfile: paymentProfileInfo.pgPaymentProfileId,
                subscriptionPlanId: getplan.id,
                userId: userData.id
            }
            await userPurchasesService.createUserPurchase(purchaseData);
            await subscribedUserService.addSubscribedUser(subscribedUserData);
        } else {
            throw new CustomError(StaticMessages.TXN_ERROR, HTTPStatusCode.BAD_REQUEST);
        }
        return res.status(HTTPStatusCode.SUCCESS).json({
            status: Constants.SUCCESS,
            message: StaticMessages.PLAN_PURCHASE_SUCCESS
        });
    } catch (error) {
        Logging.logRequestError(req, error.name);
        error = getErrorMessage(error);
        res.status(HTTPStatusCode.SUCCESS).json({
            error: error,
            status: Constants.ERROR,
            message: StaticMessages.PLAN_PURCHASE_FAILED
        });
    }
};

export default {
    createSubscriptionPlan,
    getSubscriptionPlan,
    updateSubscriptionPlan,
    deleteSubscriptionPlan,
    getSubscriptionPlanById,
    buySubscriptionPlan
}