import { Request, Response } from "express";
import { HTTPStatusCode } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import UserCharacteristicService from "./userCharacteristic.service";
import UserService from "../users/user.service";
import QuestionsService from "../questions/questions.service";
import ChoicesService from "../choices/choices.service";
import { Logging } from "../../commonServices/logging.service";

const addNewCharacteristics = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') })
                }
                resolve();
            }).catch(err => reject(err))
    });

    const checkQuestionExistOrNot = () => new Promise((resolve, reject) => {
        return QuestionsService.getQuestionByQuestionId(req.body.questionId)
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Question') })
                }
                resolve(data);
            }).catch(err => reject(err))
    });

    const checkAnswerMatchWithQuestionCategory = (question) => new Promise<void>((resolve, reject) => {
        const answers = req.body.answers;
        const matchedAnsers = answers.map(async (ans) => {
            let answerDetails = await ChoicesService.getChoicesByChoiceId(ans);
            if (!answerDetails) return reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage(`Answer with Id: ${ans}`) })
            else if (answerDetails && answerDetails.quesCategoryId !== question.quesCategoryId) {
                return reject({ status: HTTPStatusCode.CONFLICT, message: StaticMessages.CHOICE_NOT_MATCH_WITH_QUESTIONID })
            }
        });

        return Promise.all(matchedAnsers)
            .then(() => resolve())
            .catch((err) => reject(err))
    });

    const removeAllOldCharsWithSameQuestionId = () => new Promise<void>((resolve, reject) => {
        return UserCharacteristicService.deleteCharacteristicsByCondition({
            userId: authUser.id,
            questionId: parseInt(req.body.questionId)
        })
            .then(() => resolve())
            .catch(err => reject(err))
    });

    const addChars = () => new Promise((resolve, reject) => {
        const answers = req.body.answers;
        const insertDataArr = [];
        answers.map((ans) => {
            insertDataArr.push({
                userId: parseInt(authUser.id),
                customAns: req.body.customAns,
                questionId: parseInt(req.body.questionId),
                answer: parseInt(ans)
            })
        })
        return UserCharacteristicService.addMultipleCharacteristicsAtOnce(insertDataArr)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(checkQuestionExistOrNot)
        .then(checkAnswerMatchWithQuestionCategory)
        .then(removeAllOldCharsWithSameQuestionId)
        .then(addChars)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Characteristics')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getMyCharacteristics = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') });
                }
                resolve();
            }).catch(err => reject(err))
    });

    const getChars = () => new Promise((resolve, reject) => {
        return UserCharacteristicService.getCharacteristicsByUserId(authUser.id)
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt) => {
                        delete dt['Question.name'];
                        delete dt['Question.quesCategoryId'];
                        delete dt['Question.QuestionCategory.name'];
                        delete dt['Question.QuestionCategory.id'];
                        delete dt['Choice.name'];
                        return dt;
                    })
                    resolve(data);
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Characteristics') });
            })
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(getChars)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Characteristics'),
                characteristics: data
            });
        }).catch((err) => {
            Logging.logMessage(err)
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};
export default {
    addNewCharacteristics,
    getMyCharacteristics
}