import { Request, Response } from "express";
import { CommonAnswers, FolderNames, FollowStatus, FollowStatusText, HTTPStatusCode, Questions, UserInterestedIn, UserLastStatus } from "../../../utills/enums";
import { Constants } from "../../../utills/constants";
import { StaticMessages } from "../../../utills/staticMessage";
import UserPreferencesService from "./userPreferences.service";
import UserCharacteristicService from "../userCharacteristic/userCharacteristic.service";
import UserService from "../users/user.service";
import DatingPageMediaService from "../datingPageMedia/datingPageMedia.service";
import QuestionsService from "../questions/questions.service";
import ChoicesService from "../choices/choices.service";
import { Logging } from "../../commonServices/logging.service";
import moment from "moment";
import LocationService from "../locations/location.service";
import { Utils } from "../../../utills/helperMethods";
import DatePageActionService from "../datePageAction/datePageAction.service";
import DistanceMatchService from "../distanceMatch/distanceMatch.service";
import SubscribedUserService from "../subscribedUser/subscribedUser.service";
import { CustomError } from "../../../utills/types";
import getErrorObject from "../../../utills/errorStructure";
import UserPromptsService from "../userPrompts/userPrompts.service";
import UserOnlineLogsService from "../userOnlineLogs/userOnlineLogs.service";
import FollowingFollowerService from "../followingFollower/followingFollower.service";
import blockUserService from "../blockUser/blockUser.service";
import reportUserService from "../reportedUsers/reportedUsers.service";
import AgeRangesService from "../ageRanges/ageRanges.service";

const addNewPreference = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') })
                }
                resolve();
            }).catch(err => reject(err))
    });

    const checkQuestionExistOrNot = () => new Promise((resolve, reject) => {
        return QuestionsService.getQuestionByQuestionId(req.body.questionId)
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Question') })
                }
                resolve(data);
            }).catch(err => reject(err))
    });

    const checkAnswerMatchWithQuestionCategory = (question) => new Promise<void>((resolve, reject) => {
        const answers = req.body.answers;
        const matchedAnsers = answers.map(async (ans) => {
            let answerDetails = await ChoicesService.getChoicesByChoiceId(ans);
            if (!answerDetails) return reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage(`Answer with Id: ${ans}`) })
            else if (answerDetails && answerDetails.quesCategoryId !== question.quesCategoryId) {
                return reject({ status: HTTPStatusCode.CONFLICT, message: StaticMessages.CHOICE_NOT_MATCH_WITH_QUESTIONID })
            }
        });

        return Promise.all(matchedAnsers)
            .then(() => resolve())
            .catch((err) => reject(err))
    })

    const removeAllOldPrefsWithSameQuestionId = () => new Promise<void>((resolve, reject) => {
        return UserPreferencesService.deletePreferencesByCondition({
            userId: authUser.id,
            questionId: parseInt(req.body.questionId)
        })
            .then(() => resolve())
            .catch(err => reject(err))
    });

    const addPrefs = () => new Promise((resolve, reject) => {
        const answers = req.body.answers;
        const insertDataArr = [];
        answers.map((ans) => {
            insertDataArr.push({
                userId: parseInt(authUser.id),
                questionId: parseInt(req.body.questionId),
                answer: parseInt(ans)
            })
        });
        return UserPreferencesService.addMultiplePreferencesAtOnce(insertDataArr)
            .then((data) => resolve(data))
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(checkQuestionExistOrNot)
        .then(checkAnswerMatchWithQuestionCategory)
        .then(removeAllOldPrefsWithSameQuestionId)
        .then(addPrefs)
        .then(() => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getCreatedMessage('Preference')
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getMyPreferences = (req: Request, res: Response) => {
    const authUser = req['user'];
    const checkUserIsExistOrNot = () => new Promise<void>((resolve, reject) => {
        return UserService.getUserById(String(authUser.id))
            .then((data) => {
                if (!data) {
                    reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('User') });
                }
                resolve();
            }).catch(err => reject(err))
    });

    const getPrefs = () => new Promise((resolve, reject) => {
        return UserPreferencesService.getPreferencesByUserId(authUser.id)
            .then((data) => {
                if (data.length > 0) {
                    data.map((dt) => {
                        delete dt['Question.name'];
                        delete dt['Question.quesCategoryId'];
                        delete dt['Question.QuestionCategory.name'];
                        delete dt['Choice.name'];
                        return dt;
                    })
                    resolve(data);
                }
                reject({ status: HTTPStatusCode.NOTFOUND, message: StaticMessages.getNotExistMessage('Preferences') });
            })
            .catch(err => reject(err))
    });

    return checkUserIsExistOrNot()
        .then(getPrefs)
        .then((data) => {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.SUCCESS,
                message: StaticMessages.getListReturnedMessage('Preference'),
                preferences: data
            });
        }).catch((err) => {
            if (err instanceof Error) {
                return res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
                    status: Constants.ERROR,
                    message: StaticMessages.INTERNAL_SERVER_ERROR
                });
            } else {
                return res.status(err.status).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            }
        })
};

const getPreferencedUsers = async (req: Request, res: Response) => {
    try {
        const authUser = req['user'];
        let isCurrentUserLatLongFound = false;
        let sourceUserLatLong = {
            latitude: 0,
            longitude: 0
        };
        let ignoreUserIds = [authUser.id];

        const userDetail = await UserService.getUserById(String(authUser.id))
        if (!userDetail) {
            throw new CustomError(StaticMessages.getNotExistMessage('User'), HTTPStatusCode.NOTFOUND);
        }
        if (userDetail.isDocumentVerified !== 3 && userDetail.isDocumentVerified !== '3' ) { 
            throw new CustomError(StaticMessages.DOCUMENTS_NOT_VERIFIED, HTTPStatusCode.ERROR_410);
        }

        // get list of users not to be considered in getting profile preferences
        const ignoredUsersPromise = DatePageActionService.getActionsByUserId(String(authUser.id));
        const blockedUsersPromise = blockUserService.getBlockUserByUserId(authUser.id);
        const blockedByUsersPromise = blockUserService.getBlocksForUserId(authUser.id);
        const reportedUsersPromise = reportUserService.getReportedUserByMe(authUser.id);
        const reportedByUsersPromise = reportUserService.getReportedUser(authUser.id);

        const [ ignoredUsers, blockedUsers, blockedByUsers, reportedUsers, reportedByUsers ] = await Promise.all( [ ignoredUsersPromise, blockedUsersPromise, blockedByUsersPromise, reportedUsersPromise, reportedByUsersPromise ] );

        if( ignoredUsers && ignoredUsers.length ) {
            ignoredUsers.forEach((action) => ignoreUserIds.push(action.actionUserId));
        }
        if( blockedUsers && blockedUsers.length ) {
            blockedUsers.forEach((action) => ignoreUserIds.push(action.blockForUserId));
        }
        if( blockedByUsers && blockedByUsers.length ) {
            blockedByUsers.forEach((action) => ignoreUserIds.push(action.blockByUserId));
        }
        if( reportedUsers && reportedUsers.length ) {
            reportedUsers.forEach((action) => ignoreUserIds.push(action.toUserId));
        }
        if( reportedByUsers && reportedByUsers.length ) {
            reportedByUsers.forEach((action) => ignoreUserIds.push(action.userId));
        }

        // get user gender
        const userGender = userDetail.gender;
        // get user gender preferences
        const userGenderPreference = await UserPreferencesService.getGenderPreference(authUser.id);
        const genderPreference = ( userGenderPreference && userGenderPreference.answer ) ? userGenderPreference.answer : UserInterestedIn.EVERYONE;
        
        // Get the users whose gender preferences intersect with loggedIn users' gender preferences
        const preferedUsersByGender = await UserPreferencesService.getUsersWithGenderMatches(userGender, genderPreference, ignoreUserIds);

        // get the age range set by logged in user
        const ageRange = await AgeRangesService.getAgeRangeByUserId(authUser.id);
        let preferedUsersByAgeRange = [];
        if(ageRange && ageRange.minAge && ageRange.maxAge ) {
            preferedUsersByAgeRange = await UserPreferencesService.getUsersByAgeRange(ageRange, preferedUsersByGender.map((user) => user.id ));
        }
        
        // check if altleast 60% of the users are there after filtering based on age range
        if( preferedUsersByAgeRange.length / preferedUsersByGender.length < 0.6 ) {
            preferedUsersByAgeRange = preferedUsersByGender;
        }
        // convrtion to JS object format as we have to attach some properties to users
        preferedUsersByAgeRange = JSON.parse(JSON.stringify(preferedUsersByAgeRange));
        
        //commented as questions asked during signup are not to be considered yet

        // get latitude and longitude of logged in user
        const authUserLatLong = await LocationService.getLocationByUserId(String(authUser.id));
        if ( authUserLatLong &&
            ( authUserLatLong.latitude && authUserLatLong.latitude !== '' ) &&
            ( authUserLatLong.longitude && authUserLatLong.longitude !== '' )
        ) {
            isCurrentUserLatLongFound = true;
            sourceUserLatLong.latitude = parseFloat(authUserLatLong.latitude);
            sourceUserLatLong.longitude = parseFloat(authUserLatLong.longitude);
        }

        const preferedUsersByDistance = [];
        const removedUsersByDistance = [];

        const authUserDistanceMatchDetails = await DistanceMatchService.getDistanceMatchByUserId(authUser.id);
        preferedUsersByAgeRange.forEach((user) => {
            const location: any = {}
            // check if both loggedIn user and suggested user's latitude and longitude are found
            if(isCurrentUserLatLongFound &&
                user.Location &&
                user.Location.latitude &&
                user.Location.longitude)
            {
                const destUserLatLong = {
                    latitude: parseFloat(user.Location.latitude),
                    longitude: parseFloat(user.Location.longitude)
                }
                location.latitude = parseFloat(user.Location.latitude);
                location.longitude = parseFloat(user.Location.longitude);
                const distanceInKms = Utils.findDistanceAmongUsersInKms(sourceUserLatLong, destUserLatLong);
                location.distanceInKms = distanceInKms + ' KM away';
                location.distanceInMiles = Utils.findDistanceAmongUsersInMiles(sourceUserLatLong, destUserLatLong) + ' miles away';
                user.location = location;

                if ( distanceInKms > authUserDistanceMatchDetails.distanceInKms ) {
                    // keeping users, who fail distance criteria, so that they can be considered in case of insufficient result
                    removedUsersByDistance.push(user);
                    return;
                }
            } else {
                // showing static distance details in case of insufficient location details
                location.distanceInKms = 'Less than a KM away';
                location.distanceInMiles = 'Less than a mile away';
                location.latitude = 0;
                location.longitude = 0;
                user.location = location;
            }
            preferedUsersByDistance.push(user);
        });

        // check if filtered users by distance criteria yields to atleast 60% and failing so, all will be considered with updated sorting order
        if( preferedUsersByDistance.length / preferedUsersByAgeRange.length < 0.6 ) {
            preferedUsersByDistance.push(...removedUsersByDistance);
        }

        const finalResp = [];
        const prefUserMap = preferedUsersByDistance.map(async (prefUser) => {
            prefUser.userId = prefUser.id;
            delete prefUser.id;
            delete prefUser.Location;
            // getting age of user from its DOB
            prefUser['age'] = prefUser['DOB'] == null ? null : moment().diff(prefUser['DOB'].split('/').join('-'), 'years');

            prefUser.profilePicture = prefUser.profilePicture ? '/static/users/' + prefUser.userId + '/' + FolderNames.PROFILE_PICTURE + '/' + prefUser.profilePicture : "";
            prefUser.firstName = prefUser.firstName || "";
            prefUser.lastName = prefUser.lastName || "";

            // set static follow status
            prefUser.followStatus = FollowStatus.FOLLOW;
            prefUser.followStatusText = FollowStatusText.FOLLOW;

            // get user online details, medias and Prompt questions in single query
            const userOnlineLogMediaAndPrompts = await UserPreferencesService.getUserOnlineLogMediaAndPrompts(prefUser.userId);

            const userOnlineLog = userOnlineLogMediaAndPrompts.UserOnlineLogs[0];
            if ((!userOnlineLog) || (userOnlineLog && userOnlineLog.offTimestamp && userOnlineLog.offTimestamp !== 0)) {
                prefUser.onlineStatus = UserLastStatus.OFFLINE;
                prefUser.lastSeenTimestamp = userOnlineLog && userOnlineLog.offTimestamp || moment().subtract(5, 'minutes').unix();
            } else if (userOnlineLog && userOnlineLog.offTimestamp == 0) {
                prefUser.onlineStatus = UserLastStatus.ONLINE;
                prefUser.lastSeenTimestamp = null;
            }

            if (authUser.id == req.params.id) {
                prefUser.onlineStatus = UserLastStatus.ONLINE;
                prefUser.lastSeenTimestamp = null;
            }

            let datingMedia = userOnlineLogMediaAndPrompts.DatingPageMedia;
            datingMedia = datingMedia.map((media) => {
                media.mediaUrl = '/static/users/' + media.userId + '/' + FolderNames.DATING_PAGE_MEDIA + '/' + media.mediaUrl;
                delete media.userId;
                return media;
            });


            const userCharacteristics = await UserCharacteristicService.getHomeEducationWorkDetailsOfUser(prefUser.userId, [Questions.WORKS, Questions.JOB_TITLE, Questions.EDUCATION_LEVEL, Questions.COLLEGE_UNIVERSITY, Questions.HOMETOWN]);
            userCharacteristics.forEach((characteristic) => {
                if (characteristic.questionId === Questions.HOMETOWN) {
                    prefUser.works = characteristic.customAns || null;
                }
                if (characteristic.questionId === Questions.EDUCATION_LEVEL) {
                    if (characteristic.answers[0] && characteristic.answers[0].answerText !== CommonAnswers.PREFER_NOT_TO_SAY) {
                        prefUser.educationLevel = characteristic.answers[0].answerText;
                    } else {
                        prefUser.educationLevel = null;
                    }
                }
                if (characteristic.questionId === Questions.COLLEGE_UNIVERSITY) {
                    if (prefUser.educationLevel !== null) {
                        prefUser.university = characteristic.customAns || null;
                    } else {
                        prefUser.university = null;
                    }
                }
                if (characteristic.questionId === Questions.WORKS) {
                    prefUser.works = characteristic.customAns || null;
                }
                if (characteristic.questionId === Questions.JOB_TITLE) {
                    prefUser.jobTitle = characteristic.customAns || null;
                }
            });

            // setting prompts details in format frontEnd accepts
            const prefUserPrompts = JSON.parse(JSON.stringify(userOnlineLogMediaAndPrompts.UserPrompts));
            prefUserPrompts.map((prompt) => {
                prompt['userPromptsId'] = prompt.id;
                prompt['promptsCategoryId'] = prompt['PromptsQuestion']['promptsCategoryId'];
                prompt['promptsCategory'] = prompt['PromptsQuestion']['PromptsCategory']['name'];
                prompt['promptsQuestion'] = prompt['PromptsQuestion']['name'];
                delete prompt['id'];
                delete prompt['userId'];
                delete prompt['PromptsQuestion'];
                return prompt;
            });

            delete prefUser["Location.id"];
            delete prefUser["Location.longitude"];
            delete prefUser["Location.latitude"];
            delete prefUser["Location.address"];
            delete prefUser["Location.locationPrivacy"];
            delete prefUser["Location.createdAt"];
            delete prefUser["Location.updatedAt"];
            delete prefUser["Location.UserId"];
            delete prefUser["UserPreferences"];

            finalResp.push({
                userId: prefUser.userId,
                userBasicInfo: prefUser,
                datingPageMedia: datingMedia,
                prompts: prefUserPrompts,
                interests: ["Netflix"],
                socialMedia: [
                    {
                        mediaType: "FACEBOOK",
                        mediaId: "example"
                    }
                ],
                location: prefUser.location
            })
            delete prefUser["location"];
            return prefUser;
        });

        return Promise.all(prefUserMap)
            .then(() => {
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.SUCCESS,
                    message: StaticMessages.getListReturnedMessage('Preferenced Users'),
                    preferedUsers: finalResp
                });
            }).catch(err => {
                err = getErrorObject(err);
                return res.status(HTTPStatusCode.SUCCESS).json({
                    status: Constants.ERROR,
                    message: err.message
                });
            });
    } catch (err) {
        Logging.logMessage(`Error.. ${err}`);
        if (err instanceof Error) {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: StaticMessages.INTERNAL_SERVER_ERROR
            });
        } else {
            return res.status(HTTPStatusCode.SUCCESS).json({
                status: Constants.ERROR,
                message: err.message
            });
        }
    }
}

export default {
    addNewPreference,
    getMyPreferences,
    getPreferencedUsers
}
