import db from "../../../models";
import sequelize, { Op } from "sequelize";
import { DocumentStatus, Gender, UserInterestedIn } from "../../../utills/enums";
const UserPreference = db.UserPreference;
const User = db.User;
const UserOnlineLogs = db.UserOnlineLogs;
const DatingPageMedia = db.DatingPageMedia;
const UserPrompts = db.UserPrompts;

interface ansArray {
    userId: number,
    questionId: number,
    answer: number
};

const addPreferences = async (data: any) => {
    try {
        return await UserPreference.create(data);
    } catch (err: any) {
        throw err;
    }
};

const addMultiplePreferencesAtOnce = async (data: Array<ansArray>) => {
    try {
        return await UserPreference.bulkCreate(data);
    } catch (err: any) {
        throw err;
    }
};

const getPreferencesByUserId = async (userId: any) => {
    try {
        const condition = {
            where: {
                userId: String(userId)
            },
            include: [
                {
                    model: db.Question,
                    as: "Question",
                    attributes: [
                        "name",
                        "quesCategoryId"
                    ],
                    include: {
                        model: db.QuestionCategory,
                        as: "QuestionCategory",
                        attributes: ["name"],
                    }
                },
                {
                    model: db.Choice,
                    as: "Choice",
                    attributes: ["name"]
                },
            ],
            attributes: [
                ["id", "preferenceId"],
                "userId",
                "questionId",
                [sequelize.col('Question.name'), 'question'],
                [sequelize.col('Question.quesCategoryId'), 'quesCategoryId'],
                [sequelize.col('Question.QuestionCategory.name'), 'quesCategory'],
                ["answer", "answerId"],
                [sequelize.col('Choice.name'), 'answer'],
            ],
            raw: true
        };
        
        const preferences = await UserPreference.findAll(condition);
        const resp = [];
        for (let prefs of preferences) {
            if (resp.filter((v) => v.questionId == prefs.questionId).length > 0) {
                resp[resp.findIndex((fArr) => fArr.questionId == prefs.questionId)].answers.push({
                    answerId: prefs.answerId,
                    answerText: prefs['Choice.name']
                })
            } else {
                resp.push({
                    userId: prefs.userId,
                    questionId: prefs.questionId,
                    question: prefs['Question.name'],
                    quesCategoryId: prefs['Question.quesCategoryId'],
                    quesCategory: prefs['Question.QuestionCategory.name'],
                    answers: [
                        {
                            answerId: prefs.answerId,
                            answerText: prefs['Choice.name']
                        }
                    ]
                })
            }
        }
        return resp;
    } catch (err: any) {
        throw err;
    }
};

const getPreferencesByUserIdInShort = async (userId: any) => {
    try {
        return await UserPreference.findAll({
            where: {
                userId: String(userId),
                questionId: {
                    [Op.notIn]: [1, 2]
                },
                answer: {
                    [Op.notIn]: [2, 3, 4, 5, 6, 7, 8]
                }
            },
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

const getSortedUsersByQuestions = async (questionIds: any, answerIds: any, userIds: any) => {
    try {
        const users = await User.findAll({
            where: {
                id: userIds
            },
            include: [{
                model: UserPreference,
                where: {
                    questionId: questionIds,
                    answer: answerIds
                }
            }],
            attributes: [
                "id",
                "userName",
                "firstName",
                "lastName",
                "profilePicture",
                "DOB",
                "gender",
                "height",
                "heightVerified",
                "instagramToken",
                "tiktokToken",
                "spotifyToken",
                "appleMusicToken"
            ]
        });
        const sortedResult = users.sort( (a, b) => b.UserPreferences.length - a.UserPreferences.length );
        return sortedResult;
    } catch (err: any) {
        throw err;
    }
};

const getGenderPreference = async (userId: any) => {
    try {
        return await UserPreference.findOne({
            where: {
                userId: String(userId),
                questionId: 1
            },
            raw: true
        });
    } catch (err: any) {
        throw err;
    }
};

const getUsersWithGenderMatches = async (userGender: String, preferedGender: any, ignoreUserIds: any) => {
    try {
        const genderToFind = preferedGender === UserInterestedIn.MEN ? Gender.M
            : ( preferedGender === UserInterestedIn.WOMEN ? Gender.F : Gender.X);
        const userGenderNum = userGender === Gender.M ? [UserInterestedIn.MEN, UserInterestedIn.EVERYONE]
            : ( userGender === Gender.F ? [UserInterestedIn.WOMEN, UserInterestedIn.EVERYONE] : [UserInterestedIn.EVERYONE] );

        const query = {
            where: {
                gender: genderToFind,
                isActive: 1,
                isDocumentVerified: DocumentStatus.APPROVED,
                id: {
                    [Op.notIn]: ignoreUserIds
                },
            },
            include: [
                {
                    model: UserPreference,
                    as: 'UserPreferences',
                    where: {
                        questionId: 1,
                        answer: userGenderNum
                    }
                },
                {
                    model: db.Location,
                    as: 'Location'
                },
                {
                    model: db.SpotifyPlaylists,
                    as: 'SpotifyPlaylists',
                    include: [{
                        model: db.SpotifyTracks,
                        as: 'SpotifyTracks'
                    }]
                }],
            attributes: [
                'id',
                "userName",
                "firstName",
                "lastName",
                "profilePicture",
                "DOB",
                "height",
                "heightVerified",
                "instagramToken",
                "tiktokToken",
                "spotifyToken",
                "appleMusicToken"
            ]
        };

        // if (genderToFind === Gender.X) {
            delete query.where.gender;
        // }

        // if (userGenderNum[0] === UserInterestedIn.EVERYONE) {
            delete query.include[0].where;
        // }

        return await User.findAll(query);
    } catch (err: any) {
        throw err;
    }
};

const getUsersByAgeRange = async (ageRange: any, userIds: any) => {
    try {
        const currentYear = new Date().getFullYear();
        const maxYear = currentYear - ageRange.minAge;
        const minYear = currentYear - ageRange.maxAge;
        return await User.findAll({
            where: {
                id: userIds,
                birthYear : {
                    [Op.gte]: minYear,
                    [Op.lte]: maxYear,
                }
            },
            include: [{
                model: db.Location,
                as: 'Location'
            }, {
                model: db.SpotifyPlaylists,
                as: 'SpotifyPlaylists',
                include: [{
                    model: db.SpotifyTracks,
                    as: 'SpotifyTracks'
                }]
            }],
            attributes: [
                'id',
                "userName",
                "firstName",
                "lastName",
                "profilePicture",
                "DOB",
                "height",
                "heightVerified",
                "instagramToken",
                "tiktokToken",
                "spotifyToken",
                "appleMusicToken"
            ]
        });
    } catch (err: any) {
        throw err;
    }
};

const deletePreferencesByCondition = async (condition: any) => {
    try {
        return await UserPreference.destroy({
            where: condition
        });
    } catch (err: any) {
        throw err
    }
};

const getUserOnlineLogMediaAndPrompts = async (userId: any) => {
    try {
        return await User.findOne({
            where: {
                id: userId
            },
            include: [{
                model: UserOnlineLogs,
                as: 'UserOnlineLogs',
                order: [["id", "DESC"]],
                limit: 1
            }, {
                model: DatingPageMedia,
                as: 'DatingPageMedia',
                order: [['isPrimaryImage', 'DESC'], ['isDeletable', 'DESC']],
                attributes: [ "id", "mediaUrl", "mediaType", "mediaTypeText", "isDeletable", "isPrimaryImage", "userId" ]
            }, {
                model: UserPrompts,
                as: 'UserPrompts',
                include: [
                    {
                        model: db.PromptsQuestion,
                        as: "PromptsQuestion",
                        include: {
                            model: db.PromptsCategory,
                            as: "PromptsCategory"
                        }
                    }
                ]
            }],
        });
    } catch (err: any) {
        throw err
    }
};

export default {
    addPreferences,
    addMultiplePreferencesAtOnce,
    getPreferencesByUserId,
    getPreferencesByUserIdInShort,
    deletePreferencesByCondition,
    getGenderPreference,
    getUsersWithGenderMatches,
    getSortedUsersByQuestions,
    getUsersByAgeRange,
    getUserOnlineLogMediaAndPrompts
};
