import { NextFunction, Request, Response } from "express";
import Joi, { allow } from "joi";
import { Constants } from "../../../utills/constants";
import { HTTPStatusCode } from "../../../utills/enums";
import { Logging } from "../../commonServices/logging.service";
const updateUserById = (req: Request, res: Response, next: NextFunction) => {
    try {
        const schema = Joi.object().keys({
            profilePicture: Joi.binary().optional(),
            files: Joi.array().optional(),
            email: Joi.string().email().optional(),
            firstName: Joi.string().optional(),
            lastName: Joi.string().optional(),
            DOB: Joi.string().optional(),
            userName: Joi.string().optional(),
            gender: Joi.string().optional(),
            defaultNotification: Joi.string().optional(),
            profileBio: Joi.string().allow('').optional(),
            fcmToken: Joi.string().optional(),
            emailNotification: Joi.number().valid(0, 1).optional(),
            smsNotification: Joi.number().valid(0, 1).optional(),
            isLastSeenEnabled: Joi.number().valid(0, 1).optional(),
            themeColor: Joi.string().optional(),
            countryCode: Joi.string().optional(),
            phone: Joi.string().min(10).max(10).optional(),
            notificationPreference: Joi.string().valid('SMS', 'EMAIL').optional(),
            requestHeader: Joi.string().optional().description('requestHeader').error(new Error('Request Header is missing'))
        });

        if (req['files']) {
            req.body['files'] = req['files'];
        }

        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getProfileSearchValidator = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET users/search Validator");
    try {
        const schema = Joi.object({
            search: Joi.string().allow('').required().description('search: addvc').error(new Error('search is missing or incoreect')),
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const getUserList = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            type: Joi.string().allow('').default('').optional().description('type: addvc').error(new Error('type is missing or incoreect')),
            limit: Joi.string().allow('').default('20').description('limit').optional(),
            offset: Joi.string().allow('').default('0').description('offset').optional()
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const updatePassword = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            password: Joi.string().allow('').required().description('password').error(new Error('password is missing')),
            oldPassword: Joi.string().allow('').required().description('oldPassword').error(new Error('oldPassword is missing'))
        });
        const validation = schema.validate(req.body);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.body = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

const enableDisableUser = (req: Request, res: Response, next: NextFunction) => {
    Logging.logMessage("USer Validator  -> GET usersList Validator");
    try {
        const schema = Joi.object({
            id: Joi.string().required().error(new Error('id is missing')),
            isActive: Joi.number().valid(0, 1).required().error(new Error('isActive is missing'))
        });
        const validation = schema.validate(req.query);
        if (validation.error) {
            return res.status(HTTPStatusCode.BAD_REQUEST).json({
                message: validation.error.message,
                status: Constants.ERROR
            });
        }
        req.query = validation.value;
        return next();
    } catch (e) {
        Logging.logMessage(e)
        res.status(HTTPStatusCode.INTERNAL_SERVER_ERROR).json({
            message: e,
            status: Constants.ERROR
        });
        return;
    }
};

export default {
    updateUserById,
    getProfileSearchValidator,
    getUserList,
    updatePassword,
    enableDisableUser
}